/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file contours.cpp
@ingroup ExamplePrograms
@brief Obtains several features from input video frames

Uses IPE for contour simplifiyng.

@todo Detailed description should follow.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./contours --URL=<path or url to video>
\endcode

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <QVApplication>
#include <QVMPlayerCamera>
#include <QVGUI>
#include <QVImageCanvas>

#include <QVPolyline>
#include <qvdta/qvdta.h>
#include <qvip/qvip.h>

#ifndef DOXYGEN_IGNORE_THIS
class CannyOperatorWorker: public QVWorker
	{
	public:
		CannyOperatorWorker(QString name): QVWorker(name)
			{
			addProperty<double>("cannyHigh", inputFlag, 150, "High threshold for Canny operator", 50, 1000);
			addProperty<double>("cannyLow", inputFlag, 50, "Low threshold for Canny operator", 10, 500);
			addProperty<bool>("applyIPE", inputFlag, TRUE, "If we want to apply the IPE algorithm");
			addProperty<double>("paramIPE", inputFlag, 5.0, "IPE parameter (max. allowed distance to line)", 1.0, 25.0);
			addProperty<bool>("intersectLines", inputFlag, TRUE, "If we want IPE to postprocess polyline (intersecting lines)");
			addProperty<int>("minLengthContour", inputFlag, 25, "Minimal length of a contour to be considered", 1, 150);
			addProperty<int>("showNothingCannyImage", inputFlag, 0, "If we want nothing|Canny|original image to be shown",0,2);
			addProperty<bool>("showContours", inputFlag, TRUE, "If we want contours to be shown");

			addProperty< QVImage<uChar,1> >("Output image", outputFlag);
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			addProperty< QList<QVPolyline> >("Output contours", outputFlag);
			}

		void iterate()
			{
			// 0. Read input parameters
 			const double cannyHigh = getPropertyValue<double>("cannyHigh");
			const double cannyLow = getPropertyValue<double>("cannyLow");
			const bool applyIPE = getPropertyValue<bool>("applyIPE");
			const double paramIPE = getPropertyValue<double>("paramIPE");
			const bool intersectLines = getPropertyValue<bool>("intersectLines");
			const int minLengthContour = getPropertyValue<int>("minLengthContour");
			const int showNothingCannyImage = getPropertyValue<int>("showNothingCannyImage");
			const bool showContours = getPropertyValue<bool>("showContours");
			const QVImage<uChar,1> image = getPropertyValue< QVImage<uChar,3> >("Input image");
			const uInt cols = image.getCols(), rows = image.getRows();

			QVImage<sFloat> imageFloat(cols, rows), dX(cols, rows), dY(cols, rows), dXNeg(cols, rows);
			QVImage<uChar> canny(cols, rows), buffer;
		
			// 1. Convert image from uChar to sShort
			Convert(image, imageFloat);
			timeFlag("Convert image from uChar to sShort");
		
			// 2. Obtain horizontal and vertical gradients from image
			FilterSobelHorizMask(imageFloat,dY,3);
			FilterSobelVertMask(imageFloat,dX,3);
			MulC(dX, dXNeg, -1);
			timeFlag("Obtain horizontal and vertical gradients from image");
		
			// 3. Apply Canny operator
			CannyGetSize(canny, buffer);
			Canny(dXNeg, dY, canny, buffer, cannyLow,cannyHigh);
			timeFlag("Apply Canny operator");

			// 4. Get contours
			const QList<QVPolyline> contourList = getLineContoursThreshold8Connectivity(canny, 128);
			timeFlag("Get contours");

			QList<QVPolyline> outputList;
			foreach(QVPolyline contour,contourList)
				{
				if(contour.size() > minLengthContour)
					{
					if(applyIPE)
						{
						QVPolyline IPEcontour;
						IterativePointElimination(contour,IPEcontour,paramIPE,FALSE,intersectLines);
						outputList.append(IPEcontour);
						}
					else
						outputList.append(contour);
					}
				}
			timeFlag("IPE on contours");
		
			// 5. Publish resulting data
			if(showNothingCannyImage == 1)
				setPropertyValue< QVImage<uChar,1> >("Output image",canny);
			else if(showNothingCannyImage == 2)
				setPropertyValue< QVImage<uChar,1> >("Output image",image);
			else	{
				QVImage<uChar> whiteImage(cols, rows);
				Set(whiteImage,255);
				setPropertyValue< QVImage<uChar,1> >("Output image",whiteImage);
				}
			if(showContours)
				setPropertyValue< QList< QVPolyline> >("Output contours",outputList);
			else
				setPropertyValue< QList< QVPolyline> >("Output contours",QList<QVPolyline>());

			timeFlag("Publish results");
			}
	};

class ContourExtractorWorker: public QVWorker
	{
	public:
		ContourExtractorWorker(QString name): QVWorker(name)
			{
			addProperty<int>("Threshold", inputFlag,	128, "Threshold for a point to count as pertaining to a region", 0, 255);
			addProperty<int>("MinAreaIPE", inputFlag,	0, "Minimal area to keep points in the IPE algorithm", 0, 50);
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			addProperty< QList<QVPolyline> >("Internal contours", outputFlag);
			addProperty< QList<QVPolyline> >("External contours", outputFlag);
			}

		void iterate()
			{
			// 0. Read input parameters
			const QVImage<uChar,1> image = getPropertyValue< QVImage<uChar,3> >("Input image");
			const uInt	rows = image.getRows(), cols = image.getCols(),
					threshold = getPropertyValue< int >("Threshold"),
					minAreaIPE = getPropertyValue< int >("MinAreaIPE");

			timeFlag("Read input parameters");
		
			// 1. Get contours from image
			const QList<QVPolyline> contours = getConnectedSetBorderContoursThreshold(image, threshold);
			timeFlag("Get contours from image");

			// 2. Apply IPE
			QList<QVPolyline> ipeContours;

			foreach(QVPolyline polyline, contours)
				{
				QVPolyline ipePolyline;
				IterativePointElimination(polyline, ipePolyline, minAreaIPE);
				if (ipePolyline.size() > 0)
					ipeContours.append(ipePolyline);
				}

			timeFlag("IPE filtering");

			// 3. Export contours to output property
			QList<QVPolyline> internalContours, externalContours;

			foreach(QVPolyline polyline, ipeContours)
				if (polyline.direction)
					internalContours.append(polyline);
				else
					externalContours.append(polyline);

			setPropertyValue< QList< QVPolyline> >("Internal contours",internalContours);
			setPropertyValue< QList< QVPolyline> >("External contours",externalContours);
			timeFlag("Computed output contours");
			}
	};

class HarrisExtractorWorker: public QVWorker
	{
	public:
		HarrisExtractorWorker(QString name): QVWorker(name)
			{
			addProperty< int >("Points", inputFlag, 15, "window size ", 1, 100);
			addProperty< double >("Threshold", inputFlag, 1.0, "window size ", 0.0, 256.0);

			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			//addProperty< QVImage<uChar,1> >("Output image", inputFlag|outputFlag);

			addProperty< QList<QPointF> >("Corners", outputFlag);
			}

		void iterate()
			{
			// 0. Read input parameters
			const QVImage<uChar> image = getPropertyValue< QVImage<uChar,3> >("Input image");
			const double threshold = getPropertyValue<double>("Threshold");
			const int pointNumber = getPropertyValue<int>("Points");
			timeFlag("grab Frame");
		
			// 1. Obtain corner response image.
			QVImage<sFloat>	cornerResponseImage(image.getRows(), image.getCols());
			FilterHessianCornerResponseImage(image, cornerResponseImage);
			timeFlag("Corner response image");

			// 2. Local maximal filter.
			QList<QPointF> hotPoints = GetMaximalResponsePoints3(cornerResponseImage, threshold);
			timeFlag("Local maximal filter");

			// 3. Output resulting data.
			//setPropertyValue< QVImage<uChar,1> >("Output image", image);
			setPropertyValue< QList<QPointF> >("Corners", hotPoints.mid(MAX(0,hotPoints.size() - pointNumber)));
			}
	};

#include <QVMSER>
class MSERExtractorWorker: public QVWorker
	{
	private:
		const QVImage<uChar> negateImage(const QVImage<uChar> image) const
			{
			const uInt rows = image.getRows(), cols = image.getCols();
			QVImage<uChar> notImage(cols, rows);
			for (uInt col = 0; col < cols; col++)
				for (uInt row = 0; row < rows; row++)
					notImage(col, row) = 255 - image(col, row);

			return notImage;
			}

	public:
		MSERExtractorWorker(QString name): QVWorker(name)
			{
			addProperty<int>("Delta", inputFlag, 10, "MSER parameter, as seen in the paper.", 1, 128);
			addProperty<int>("minAreaMSER", inputFlag, 10, "MSER with area lesser than this value are discarted.", 1, 100*100);
			addProperty<int>("maxAreaMSER", inputFlag, 10000, "MSER with area greater than this value are discarted.", 1, 1000*1000);
			addProperty<double>("diffAreaThreshold", inputFlag, 0.01, "Proportion of the difference in areas to be considered different", 0.0, 1.0);
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			addProperty< QList<QVPolyline> >("MSER contours", outputFlag);
			}

		void iterate()
			{
			// 0. Read parameters
			const int delta = getPropertyValue<int>("Delta"),
				minArea = getPropertyValue<int>("minAreaMSER"),
				maxArea = getPropertyValue<int>("maxAreaMSER");
			const double diffAreaThreshold = getPropertyValue<double>("diffAreaThreshold");
			const QVImage<uChar> image = getPropertyValue< QVImage<uChar,3> >("Input image");
			const QVImage<uChar> notImage = negateImage(image);

			timeFlag("Read parameters");

			// 1. Apply MSER
			QList<QVMSER> MSERListLow, MSERListHigh;

			getMSER(image, MSERListLow, delta, minArea, maxArea, diffAreaThreshold);
			timeFlag("MSER Low");

			getMSER(notImage, MSERListHigh, delta, minArea, maxArea, diffAreaThreshold);
			timeFlag("MSER High");

			// 2. Publish resulting MSER's
			QList< QVPolyline > polylineMSERList;
			getMSERContours(image, MSERListLow, polylineMSERList);
			getMSERContours(notImage, MSERListHigh, polylineMSERList);

			setPropertyValue< QList<QVPolyline> >("MSER contours", polylineMSERList);
			timeFlag("Publish resulting images");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		"Example program for QVision library. Obtains several features from input video frames."
		);

	CannyOperatorWorker cannyWorker("Canny Operator Worker");
	ContourExtractorWorker contoursWorker("Contours Extractor Worker");
	HarrisExtractorWorker cornersWorker("Corners Worker");
	MSERExtractorWorker mserWorker("MSER Worker");

	QVMPlayerCamera camera("Video");

	camera.link(&cannyWorker,"Input image");
	camera.link(&contoursWorker,"Input image");
	camera.link(&cornersWorker,"Input image");
	camera.link(&mserWorker,"Input image");

	QVGUI interface;

	QVImageCanvas cannyCanvas("Canny");
	cannyCanvas.linkProperty(cannyWorker,"Output image");
	cannyCanvas.linkProperty(cannyWorker,"Output contours");

	QVImageCanvas contourCanvas("Contours");
	contourCanvas.linkProperty(contoursWorker, "Input image");
	contourCanvas.linkProperty(contoursWorker,"Internal contours", Qt::red);
	contourCanvas.linkProperty(contoursWorker,"External contours", Qt::blue);

	QVImageCanvas cornersCanvas("Corners");
	cornersCanvas.linkProperty(cornersWorker, "Input image");
	cornersCanvas.linkProperty(cornersWorker,"Corners", Qt::blue, false);

	QVImageCanvas mserCanvas("MSER Regions");
	mserCanvas.linkProperty(mserWorker, "Input image");
	mserCanvas.linkProperty(mserWorker,"MSER contours", Qt::red);

	return app.exec();
	}

#endif
