/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvdta.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVDTA_H
#define QVDTA_H

#include <QList>
#include <QMap>
#include <QVector>
#include <QPoint>
#include <iostream>
#include <qvcore/qvimage.h>

/*!
@brief Computer Vision Data types and Algorithms.

See \ref Structure for a detailed explanation of the structure of this package.

*/

/// @brief Sorts pixels in an image, given their gray-scale value.
/// @ingroup qvip
///
/// This function obtains a QVector, that maps any gray-scale values (0..255) to the set of QPoints that hold that gray-scale
/// value, in a given image. It can be used for very fast pixel sorting.
///
/// The function itself is based in \htmlonly <a href="http://en.wikipedia.org/wiki/Counting_sort">Counting Sort algorithm</a> \endhtmlonly, so the cost of this function is O(number_of_columns * number_of_rows).
///
/// @param img source image to obtain sorted lists of QPoints.
/// @return A QVector, indexed by the gray-scale value (0-255), to a QVector of QPoints, containing the corresponding points to the pixels that hold that gray-scale value in the image.
QVector< QVector< QPoint > > CountingSort(const QVImage<uChar, 1> &img);

#ifndef DOXYGEN_IGNORE_THIS
/// @ingroup qvdta
/// @todo document this
template <typename T1, typename T2> QList< QPair<T1,T2> > joinPairList(const QList<T1> &list1, const QList<T2> &list2)
	{
	QList< QPair<T1,T2> > result;
	for (int i = 0; i < MIN(list1.size(),list2.size()); i++)
		result.append(QPair<T1,T2>(list1.at(i), list2.at(i)));
	return result;
	}

/// @ingroup qvdta
/// @todo document this
template <typename T1, typename T2> QList<T1> getFirstPairList(const QList< QPair<T1,T2> > &list)
	{
	QList<T1> result;
	QPair<T1,T2> pair;
	foreach(pair, list)
		result.append(pair.first);
	return result;
	}

/// @ingroup qvdta
/// @todo document this
template <typename T1, typename T2> QList<T2> getSecondPairList(const QList< QPair<T1,T2> > &list)
	{
	QList<T2> result;
	QPair<T1,T2> pair;
	foreach(pair, list)
		result.append(pair.second);
	return result;
	}

/// @ingroup qvdta
/// @todo document this
template <typename T1, typename T2> QList<T1> convertList(QList<T2> &list)
	{
	QList<T1> result;
	foreach(T2 element, list)
		result.append(element);
	return result;
	}

#endif

/// @todo document this
/// @ingroup qvip
QList<QPointF> GetMaximalResponsePoints1(const QVImage<sFloat> &cornerResponseImage, const double threshold = 1.0);

/// @todo document this
/// @ingroup qvip
QList<QPointF> GetMaximalResponsePoints3(const QVImage<sFloat> &cornerResponseImage, const double threshold = 1.0);

/// @todo document this
/// @ingroup qvip
QList< QPair<sFloat, QPointF> > GetMaximalResponsePoints1bis(const QVImage<sFloat> &cornerResponseImage, const double threshold = 1.0);

/// @todo document this
/// @ingroup qvip
QList< QPair<sFloat, QPointF> > GetMaximalResponsePoints3bis(const QVImage<sFloat> &cornerResponseImage, const double threshold = 1.0);

#endif
