/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvgui.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVGUI_H
#define QVGUI_H

#include <QWidget>
#include <QMap>
#include <QCloseEvent>

#include <QToolBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGroupBox>

/// @deprecated qvgui.h should not include qvimagecanvas.h
#include <qvgui/qvimagecanvas.h>
#include <qvgui/qvplot.h>
#include <qvcore/qvcamera.h>
#include <qvcore/qvworker.h>

class QVImageCanvas;

/*!
@class QVGUI qvgui.h qvgui/qvgui.h
@brief Main user interface widget for QVision applications.

This class can be used to create an interactive widget that will offer to the user the possibility to control the execution of the different workers registered in the application, the flow of the input video camera objects, and modify the scalar input parameters defined and not linked at the workers (see @ref DynamicProperties ).

This is a sample snapshot for a QVGUI window:

\image html qvisioninterface_rotoscoper.png

To be correctly initialized a sole instance object of the QVGUI class should be created after the QVApplication object in the <i>main</i> function, and before the call to <i>exec()</i>. For a general usage of this class in a real QVision application, see @ref ProgrammingModel section.

\section QVGUIWidgets QVGUI widgets.
There are two widget areas in the QVGUI window:

 - <b>Control</b> area: contains a <i>camera widget</i> and a <i>worker widget</i> for each camera and worker object created in the application, respectively. These widgets can be used to stop, resume, and pause the execution and video input flow for the workers and the cameras of the application, amongst other things.
 - <b>Workers input</b> area: contains a set of widgets allowing the user to modify the values of the parameters for the workers created by the application.

\section CameraWidget Camera widgets.
For every camera object registered in the system there is a widget in the interface, known as the camera widget, that contains buttons to control its input flow. An example of it is depicted below:

\image html qvisioninterface_rotoscoper_camera.png

Follows a description of the buttons and their functionality:

<table>
<tr><td><b>Pause button</b> </td> <td width="5%"> @image html qvisioninterface_pause.png </td>
<td>This button stops the camera from publishing new frames from the video source. If the camera is on real-time mode it will keep reading frames, but won't send them to the workers connected to it.</td></tr>

<tr><td><b>Resume button</b> </td> <td>  @image html qvisioninterface_play.png </td>
<td>This button resumes grabbing of the images that the registered workers read from the cameras.</td></tr>

<tr><td><b>Step button</b> </td> <td>  @image html qvisioninterface_step.png </td>
<td>When the camera is paused, this button makes it read the next frame in the video input, but keeps the camera paused.</td></tr>

<tr><td><b>Stop button</b> </td> <td>  @image html qvisioninterface_stop.png </td>
<td>This button stops the camera from reading frames from the input source.</td></tr>

<tr><td><b>Camera extended controls button</b> </td> <td>  @image html qvisioninterface_cameramaximize.png </td>
<td>This button opens a new window, with extended control functionality for the camera object, including buttons to control the playing speed.
Also, it displays the following information: fps, reproduction speed, size of the video images, seconds elapsed (actual/total) and frames grabbed (read from the video source) / read (sent to the workers).
@image html qvisioninterface_cameraextended.png
</td></tr>

</table>

\section WorkerWidget Worker widgets.
As for the cameras, every worker object registered in the system has its own widget in the interface. It contains several buttons that control its execution. An example of it is depicted below:

\image html qvisioninterface_rotoscoper_cannyworker.png

You can see a preview plot of CPU usage, and some buttons. Execution for the workers can be paused, resumed, stepped, and stopped with them, as if workers were cameras.

\note CPU stat plot depends on execution time between time flags, so when two or more workers compete for one CPU, times can differ than the used when each worker is run in its own CPU.

Follows a list with the detailed description of the buttons and their functionality:

<table>
<tr><td><b>Pause button</b> </td> <td width="5%"> @image html qvisioninterface_pause.png </td>
<td>This button suspends the processing of the worker. Every worker synchronized with it will be stopped as well, until the resume button is pushed.</td></tr>

<tr><td><b>Resume button</b> </td> <td> @image html qvisioninterface_play.png </td>
<td>If the worker is paused, pushing this button resumes its execution.</td></tr>

<tr><td><b>Step button</b> </td> <td> @image html qvisioninterface_step.png </td>
<td>This button can be pushed when a worker is paused to make it process it its inputs one time. The worker will still be paused after that.</td></tr>

<tr><td><b>Stop button</b> </td> <td> @image html qvisioninterface_stop.png </td>
<td>This button finish the processing of the worker. Its properties will be unlinked and with a freezed value, so any worker connected to it will read the same values from then on.</td></tr>

<tr><td><b>Cpu statistics button</b> </td> <td> @image html qvisioninterface_cpustats.png </td>
<td>This button opens a detailed cpu usage plot for the worker. It shows a window with the cpu time statistical plot of the time flags defined in the <i>iterate()</i> function of the worker. You can see an example of this plot in the following figure:<p />
@image html qvisioninterface_cpustat_canny.png </td></tr><br>
<p>Function @ref QVWorker::timeFlag() can be used in the body of the @ref QVWorker::iterate() function to configure a time flag in the execution of a worker.</p>
</table>

\section ParamInspectorWidget Workers input area.
This side of the graphical user interface contains a tab for each worker created by the application. Each tab shows a set of widgets offering control for the worker's input parameters. The user can modify the values of these input parameters at will in execution time.

Each of these widgets is connected to each input property (see section @ref DynamicProperties) from the set of created workers 

@ingroup qvgui
*/
class QVGUI: public QWidget
{
	Q_OBJECT
public:	
	//Ui::QVisionForm form;

	/// @brief Constructs a QVGUI object, given the QVision object which work (properties, result images, 
	/// etc...) will be inspected.
	/// @param name identifier for the object to be created.
	QVGUI(QWidget *parent = 0);
	~QVGUI();

	void closeEvent(QCloseEvent *event);

	#ifndef DOXYGEN_IGNORE_THIS
	// Adds controls for a QVCamera object to the interface, allowing the user to control and inspect the state of
	// the camera.
	// param camera QVCamera object which will be controlled
	void add(QVCamera *camera);

	// Adds a new QVPropertyContainer, which double, integer and boolean properties will be added to the  interface to be
	// controlled in real time by the final user
	// the camera.
	// param name Identification name of the QVPropertyContainer in the interface
	// param object QVPropertyContainer object which property controls will be shown
	void add(QVWorker * worker);
	//bool setVisible(QVWorker *worker,QString prop_orig,float low=0.0,float high=255.0);
	#endif

signals:
	/// @brief Emited when the image canvas window closes.
	///
	void closed();
		
private:
	QHBoxLayout *hboxlayout;
	QWidget *leftwidget;
	QToolBox *toolbox;
	QGroupBox *qgroupbox;
	QVBoxLayout *leftvboxlayout;
	QVBoxLayout *groupboxlayout;
	QList<QVImageCanvas*> qvimagecanvaslist;
};

#endif
