/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvipp.h
/// @brief File from the QVision library.
/// @author Copyright (C) PARP Research Group. University of Murcia, Spain.

#ifndef QVIPP_H
#define QVIPP_H

#include <qvcore/qvimage.h>
#include <QMap>

class QVMatrix;
class QVVector;

/// @brief Copies pixel values between two images.
/// @ingroup qvipp
///
/// This function copies data from a source image ROI src to the destination image ROI dest.
///
/// @ingroup qvipp
/// @param src source image.
/// @param dest destination image.
void Copy(const QVImage<uChar, 1> &src, QVImage<uChar, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<uShort, 1> &src, QVImage<uShort, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sShort, 1> &src, QVImage<sShort, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sInt, 1> &src, QVImage<sInt, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sFloat, 1> &src, QVImage<sFloat, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sShort, 3> &src, QVImage<sShort, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sInt, 3> &src, QVImage<sInt, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy pixel values between two images.
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sFloat, 3> &src, QVImage<sFloat, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a one-channel image to a multi-channel image
/// @ingroup qvipp
///
/// This function copy data from a source 1-channel image ROI src, to each of the channels of the destination 3-channel image ROI 
/// dest.
///
/// @param src source image.
/// @param dest destination image.
void Copy(const QVImage<uChar, 1> &src, QVImage<uChar, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a one-channel image to a multi-channel image
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<uShort, 1> &src, QVImage<uShort, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a one-channel image to a multi-channel image
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<sShort, 1> &src, QVImage<sShort, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a one-channel image to a multi-channel image
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<sInt, 1> &src, QVImage<sInt, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a one-channel image to a multi-channel image
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<sFloat, 1> &src, QVImage<sFloat, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Composes color image from separate channel images
/// @ingroup qvipp
///
/// This function copies data from a source 1-channel image ROI src, to each of the channels of the destination 3-channel image ROI 
/// dest.
///
/// @param src1 source image for first channel.
/// @param src2 source image for second channel.
/// @param src3 source image for third channel.
/// @param dest destination image.
void Copy(const QVImage<uChar, 1> &src1, const QVImage<uChar, 1> &src2, const QVImage<uChar, 1> &src3, QVImage<uChar, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Composes color image from separate channel images
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, const QVImage<uChar, 1> &, const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<uShort, 1> &src1, const QVImage<uShort, 1> &src2, const QVImage<uShort, 1> &src3, QVImage<uShort, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Composes color image from separate channel images
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, const QVImage<uChar, 1> &, const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<sShort, 1> &src1, const QVImage<sShort, 1> &src2, const QVImage<sShort, 1> &src3, QVImage<sShort, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Composes color image from separate channel images
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, const QVImage<uChar, 1> &, const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<sInt, 1> &src1, const QVImage<sInt, 1> &src2, const QVImage<sInt, 1> &src3, QVImage<sInt, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Composes color image from separate channel images
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 1> &, const QVImage<uChar, 1> &, const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<sFloat, 1> &src1, const QVImage<sFloat, 1> &src2, const QVImage<sFloat, 1> &src3, QVImage<sFloat, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a selected channel from a 3-channel image to a 1-channel image
/// @ingroup qvipp
///
/// This function converts an RGB image to gray scale.
///
/// @param src source image.
/// @param dest destination image.
/// @param channel number of channel to copy.
void Copy(const QVImage<uChar, 3> &src, QVImage<uChar, 1> &dest, uChar channel, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a selected channel from a 3-channel image to a 1-channel image
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 3> &, QVImage<uChar, 1> &, uChar)
void Copy(const QVImage<uShort, 3> &src, QVImage<uShort, 1> &dest, uChar channel, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a selected channel from a 3-channel image to a 1-channel image
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 3> &, QVImage<uChar, 1> &, uChar)
void Copy(const QVImage<sShort, 3> &src, QVImage<sShort, 1> &dest, uChar channel, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a selected channel from a 3-channel image to a 1-channel image
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 3> &, QVImage<uChar, 1> &, uChar)
void Copy(const QVImage<sInt, 3> &src, QVImage<sInt, 1> &dest, uChar channel, QPoint destROIOffset = QPoint(0,0));

/// @brief Copy a selected channel from a 3-channel image to a 1-channel image
/// @ingroup qvipp
/// @see Copy(const QVImage<uChar, 3> &, QVImage<uChar, 1> &, uChar)
void Copy(const QVImage<sFloat, 3> &src, QVImage<sFloat, 1> &dest, uChar channel, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of two images using a specified compare operation.
/// @ingroup qvipp
///
/// This function compares the corresponding pixels of ROI in two source images src1, src2 using the ippCmpOp compare operation, 
/// and writes the results to a 1-channel uChar image dest.
///
/// If the result of the compare is true, the corresponding output pixel is set to an IPP_MAX_8U value; otherwise, it is set to 0.
///
/// @param src1 first operand in comparison image.
/// @param src2 second operand in comparison image.
/// @param dest image to store comparison value pixels.
/// @param ippCmpOp Compare operation to be used for comparing the pixel values. Must be one of these enumerated values:
///	- ippCmpLess,		for less comparison
///	- ippCmpLessEq,		for less or equal comparison
///	- ippCmpEq,		for equal comparison
///	- ippCmpGreaterEq,	for greater or equal comparison
///	- ippCmpGreater,	for greater comparison
void Compare(const QVImage<uChar> &src1, const QVImage<uChar> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of two images using a specified compare operation.
/// @ingroup qvipp
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<uShort> &src1, const QVImage<uShort> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of two images using a specified compare operation.
/// @ingroup qvipp
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<sShort> &src1, const QVImage<sShort> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of two images using a specified compare operation.
/// @ingroup qvipp
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of two images using a specified compare operation.
/// @ingroup qvipp
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of two images using a specified compare operation.
/// @ingroup qvipp
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<uShort,3> &src1, const QVImage<uShort,3> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of two images using a specified compare operation.
/// @ingroup qvipp
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of two images using a specified compare operation.
/// @ingroup qvipp
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of a source image to a given value using a specified compare operation.
/// @ingroup qvipp
///
/// This function compares pixels of the each channel of the source image ROI src to a given
/// value specified for each channel using the ippCmpOp compare operation, and writes the
/// results to a 1-channel Ipp8u image dest.
///
/// If the result of the compare is true, that is all pixels of all channels satisfy to the specified condition,
/// then the corresponding output pixel is set to an IPP_MAX_8U value; otherwise, it is set to 0.
///
/// @param src image to compare pixel values.
/// @param dest image to store pixel comparison values.
/// @param constant value to compare each pixel value.
/// @param ippCmpOp Compare operation to be used for comparing the pixel values. Must be one of these enumerated values:
///	- ippCmpLess,		for less comparison
///	- ippCmpLessEq,		for less or equal comparison
///	- ippCmpEq,		for equal comparison
///	- ippCmpGreaterEq,	for greater or equal comparison
///	- ippCmpGreater,	for greater comparison
void CompareC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Compares pixel values of a source image to a given value using a specified compare operation.
/// @ingroup qvipp
/// @see CompareC(const QVImage<uChar> &, QVImage<uChar> &, uChar, IppCmpOp ippCmpOp)
void CompareC(const QVImage<sFloat> &src, QVImage<uChar> &dest, sFloat constant, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
///
/// This function converts pixel values in the source image ROI pSrc to a different bit depth
/// and writes them to the destination image ROI pDst. No scaling is done.
///
/// @param src image to convert bit depth.
/// @param dest image to store resulting bit depth image conversion.
void Convert(const QVImage<uChar, 1> &src, QVImage<uShort, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uChar, 1> &src, QVImage<sShort, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uChar, 1> &src, QVImage<sInt, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uChar, 1> &src, QVImage<sFloat, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uShort, 1> &src, QVImage<uChar, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uShort, 1> &src, QVImage<sInt, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uShort, 1> &src, QVImage<sFloat, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<sShort, 1> &src, QVImage<uChar, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<sShort, 1> &src, QVImage<sInt, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<sShort, 1> &src, QVImage<sFloat, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<sInt, 1> &src, QVImage<uChar, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uChar, 3> &src, QVImage<uShort, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uChar, 3> &src, QVImage<sShort, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uChar, 3> &src, QVImage<sInt, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uChar, 3> &src, QVImage<sFloat, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uShort, 3> &src, QVImage<uChar, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uShort, 3> &src, QVImage<sInt, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<uShort, 3> &src, QVImage<sFloat, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<sShort, 3> &src, QVImage<uChar, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<sShort, 3> &src, QVImage<sInt, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<sShort, 3> &src, QVImage<sFloat, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from one bit depth to another.
/// @ingroup qvipp
/// @see Convert(const QVImage<uChar, 1> &, QVImage<uShort, 1> &)
void Convert(const QVImage<sInt, 3> &src, QVImage<uChar, 3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @ingroup qvipp
///
/// This function converts pixel values in the source image ROI pSrc to a different bit depth
/// and writes them to the destination image ROI pDst. No scaling is done.
///
/// Rounding defined by roundMode is performed, and the result is saturated to the destination data type range.
///
/// @param src image to convert bit depth.
/// @param dest image to store resulting bit depth image conversion.
/// @param roundMode specified round mode. Possible values for the enumeration are:
///	- ippRndZero. Specifies that floating-point values must be truncated toward zero.
///	- ippRndNear. Specifies that floating-point values must be rounded to the nearest integer.
void Convert(const QVImage<sFloat, 1> &src, QVImage<uChar, 1> &dest, IppRoundMode roundMode = ippRndNear, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @ingroup qvipp
/// @see Convert(const QVImage<sFloat, 1> &, QVImage<uChar, 1> &, IppRoundMode)
void Convert(const QVImage<sFloat, 1> &src, QVImage<uShort, 1> &dest, IppRoundMode roundMode = ippRndNear, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @ingroup qvipp
/// @see Convert(const QVImage<sFloat, 1> &, QVImage<uChar, 1> &, IppRoundMode)
void Convert(const QVImage<sFloat, 1> &src, QVImage<sShort, 1> &dest, IppRoundMode roundMode = ippRndNear, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @ingroup qvipp
/// @see Convert(const QVImage<sFloat, 1> &, QVImage<uChar, 1> &, IppRoundMode)
void Convert(const QVImage<sFloat, 3> &src, QVImage<uChar, 3> &dest, IppRoundMode roundMode = ippRndNear, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @ingroup qvipp
/// @see Convert(const QVImage<sFloat, 1> &, QVImage<uChar, 1> &, IppRoundMode)
void Convert(const QVImage<sFloat, 3> &src, QVImage<uShort, 3> &dest, IppRoundMode roundMode = ippRndNear, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @ingroup qvipp
/// @see Convert(const QVImage<sFloat, 1> &, QVImage<uChar, 1> &, IppRoundMode)
void Convert(const QVImage<sFloat, 3> &src, QVImage<sShort, 3> &dest, IppRoundMode roundMode = ippRndNear, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts an RGB image to gray-scale using fixed transform coefficients.
/// @ingroup qvipp
///
/// This function converts an RGB image to gray scale (see Figure 6-18) using the following basic equation to compute luminance
/// from nonlinear gamma-corrected red, green, and blue values:
/// \verbatim Y = 0.299 * R + 0.587 * G + 0.114 * B \endverbatim
///
/// Note that the transform coefficients conform to the standard for the NTSC red, green, and blue CRT phosphors.
///
/// @param src RGB image to convert to gray scale.
/// @param dest resulting gray-scale image.
void RGBToGray(const QVImage<uChar, 3> &src, QVImage<uChar, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts an RGB image to gray-scale using fixed transform coefficients.
/// @ingroup qvipp
/// @see RGBToGray(const QVImage<uChar, 3> &, QVImage<uChar, 1> &)
void RGBToGray(const QVImage<uShort, 3> &src, QVImage<uShort, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Converts an RGB image to gray-scale using fixed transform coefficients.
/// @ingroup qvipp
/// @see RGBToGray(const QVImage<uChar, 3> &, QVImage<uChar, 1> &)
void RGBToGray(const QVImage<sShort, 3> &src, QVImage<sShort, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/*/// @brief Converts an RGB image to gray-scale using fixed transform coefficients.
/// @ingroup qvipp
/// @see RGBToGray(const QVImage<uChar, 3> &, QVImage<uChar, 1> &)
void RGBToGray(const QVImage<sInt, 3> &src, QVImage<sInt, 1> &dest, QPoint destROIOffset = QPoint(0,0));*/

/// @brief Converts an RGB image to gray-scale using fixed transform coefficients.
/// @ingroup qvipp
/// @see RGBToGray(const QVImage<uChar, 3> &, QVImage<uChar, 1> &)
void RGBToGray(const QVImage<sFloat, 3> &src, QVImage<sFloat, 1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs thresholding of pixel values in an image buffer.
/// @ingroup qvipp
///
/// This function thresholds pixels in the source image src using the specified level threshold.
/// Pixel values in the source image are compared to the threshold value using the ippCmpOp comparison operation.
///
/// If the result of the compare is true, the corresponding output pixel is set to the threshold value.
/// Otherwise, it is set to the source pixel value.
///
/// @param src image to threshold.
/// @param dest image to store thresholded image.
/// @param constant value to compare each pixel value.
/// @param ippCmpOp Compare operation to be used for thresholding the pixel values. Must be one of these enumerated values:
///	- ippCmpLess,		for less comparison
///	- ippCmpLessEq,		for less or equal comparison
///	- ippCmpEq,		for equal comparison
///	- ippCmpGreaterEq,	for greater or equal comparison
///	- ippCmpGreater,	for greater comparison
void Threshold(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs thresholding of pixel values in an image buffer.
/// @ingroup qvipp
/// @see Threshold(const QVImage<uChar> &, QVImage<uChar> &, uChar, IppCmpOp ippCmpOp)
void Threshold(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs thresholding of pixel values in an image buffer.
/// @ingroup qvipp
/// @see Threshold(const QVImage<uChar> &, QVImage<uChar> &, uChar, IppCmpOp ippCmpOp)
void Threshold(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant, IppCmpOp ippCmpOp = ippCmpEq, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs inplace (dest=src) thresholding of pixel values in an image buffer.
/// @ingroup qvipp
/// @see Threshold(const QVImage<uChar> &, QVImage<uChar> &, uChar, IppCmpOp ippCmpOp)
void Threshold(QVImage<uChar> &src_dest, uChar constant, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Performs inplace (dest=src) thresholding of pixel values in an image buffer.
/// @ingroup qvipp
/// @see Threshold(const QVImage<sShort> &, QVImage<sShort> &, sShort, IppCmpOp ippCmpOp)
void Threshold(QVImage<sShort> &src_dest, sShort constant, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Performs inplace (dest=src) thresholding of pixel values in an image buffer.
/// @ingroup qvipp
/// @see Threshold(const QVImage<sFloat> &, QVImage<sFloat> &, sFloat, IppCmpOp ippCmpOp)
void Threshold(QVImage<sFloat> &src_dest, sFloat constant, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Performs thresholding of pixel values in an image buffer. Pixels that satisfy the compare condition are set to a specified value.
/// @ingroup qvipp
///
/// This function thresholds pixels in the source image src using the specified level
/// threshold. Pixel values in the source image are compared to the threshold value using
/// the ippCmpOp comparison operation. If the result of the compare is true, the corresponding
/// output pixel is set to the specified value. Otherwise, it is set to the source pixel value.
///
/// @param src image to threshold.
/// @param dest image to store thresholded image.
/// @param threshold value to compare each pixel value.
/// @param value value to set if threshold condition is true.
/// @param ippCmpOp Compare operation to be used for thresholding the pixel values. Must be one of these enumerated values:
///	- ippCmpLess,		for less comparison
///	- ippCmpLessEq,		for less or equal comparison
///	- ippCmpEq,		for equal comparison
///	- ippCmpGreaterEq,	for greater or equal comparison
///	- ippCmpGreater,	for greater comparison
void Threshold_Val(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar threshold, uChar value, IppCmpOp ippCmpOp, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs thresholding of pixel values in an image buffer. Pixels that satisfy the compare condition are set to a specified value.
/// @ingroup qvipp
/// @see Threshold_Val(const QVImage<uChar> &, QVImage<uChar> &, uChar, uChar, IppCmpOp)
void Threshold_Val(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat threshold, sFloat value, IppCmpOp ippCmpOp, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs inplace (dest=src) thresholding of pixel values in an image buffer. Pixels that satisfy the compare condition are set to a specified value.
/// @ingroup qvipp
/// @see Threshold_Val(const QVImage<uChar> &, QVImage<uChar> &, uChar, IppCmpOp ippCmpOp)
void Threshold_Val(QVImage<uChar> &src_dest, uChar threshold, uChar value, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Performs inplace (dest=src) thresholding of pixel values in an image buffer. Pixels that satisfy the compare condition are set to a specified value.
/// @ingroup qvipp
/// @see Threshold_Val(const QVImage<sShort> &, QVImage<sShort> &, sShort, IppCmpOp ippCmpOp)
void Threshold_Val(QVImage<sShort> &src_dest, sShort threshold, sShort value, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Performs inplace (dest=src) thresholding of pixel values in an image buffer. Pixels that satisfy the compare condition are set to a specified value.
/// @ingroup qvipp
/// @see Threshold_Val(const QVImage<sFloat> &, QVImage<sFloat> &, sFloat, IppCmpOp ippCmpOp)
void Threshold_Val(QVImage<sFloat> &src_dest, sFloat threshold, sFloat value, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Performs double thresholding of pixel values in an image buffer.
/// @ingroup qvipp
/// This function thresholds pixels in the source image pSrc using two specified levels
/// thresholdLT and thresholdGT. Pixel values in the source image are compared to these
/// levels. If the pixel value is less than thresholdLT, the corresponding output pixel is set to
/// valueLT. If the pixel value is greater than thresholdGT, the output pixel is set to
/// valueGT. Otherwise, it is set to the source pixel value.
/// The value of thresholdLT should be less than or equal to thresholdGT.
///
/// @param src image to threshold.
/// @param dest image to store thresholded image.
/// @param thresholdLT lower value to compare each pixel value.
/// @param valueLT lower value to set if threshold condition is true.
/// @param thresholdGT higher value to compare each pixel value.
/// @param valueGT higher value to set if threshold condition is true.
void Threshold_LTValGTVal(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar thresholdLT, uChar valueLT, uChar thresholdGT, uChar valueGT, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs double thresholding of pixel values in an image buffer.
/// @ingroup qvipp
/// @see Threshold_LTValGTVal(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar thresholdLT, uChar valueLT, uChar thresholdGT, uChar valueGT, QPoint destROIOffset = QPoint(0,0));
void Threshold_LTValGTVal(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat thresholdLT, sFloat valueLT, sFloat thresholdGT, sFloat valueGT, QPoint destROIOffset = QPoint(0,0));

/// @brief Sets pixels of an image ROI to a value.
/// @ingroup qvipp
///
/// This function sets pixels in the destination image ROI img to a constant value.
///
/// @param img destination image to set.
/// @param constant value to set pixels in image ROI.
void Set(QVImage<uChar> &img, uChar constant);

/// @brief Sets pixels of an image ROI to a value.
/// @ingroup qvipp
/// @see Set(QVImage<uChar> &, uChar)
void Set(QVImage<uShort> &img, uShort constant);

/// @brief Sets pixels of an image ROI to a value.
/// @ingroup qvipp
/// @see Set(QVImage<uChar> &, uChar)
void Set(QVImage<sShort> &img, sShort constant);

/// @brief Sets pixels of an image ROI to a value.
/// @ingroup qvipp
/// @see Set(QVImage<uChar> &, uChar)
void Set(QVImage<sInt> &img, sInt constant);

/// @brief Sets pixels of an image ROI to a value.
/// @ingroup qvipp
/// @see Set(QVImage<uChar> &, uChar)
void Set(QVImage<sFloat> &img, sFloat constant);

/// @brief Sets pixels of a 3-channel image ROI to a value respective constant value, one for each channel.
/// @ingroup qvipp
///
/// This function sets pixels in the destination 3-channel image ROI img to a constant value. Each channel can be set to
/// a different constant value, with a different parameter value.
///
/// @param img destination image to set.
/// @param red value to set pixels in first channel of image ROI.
/// @param green value to set pixels in second channel of image ROI.
/// @param blue value to set pixels in third channel of image ROI.
void Set(QVImage<uChar, 3> &img, uChar red, uChar green, uChar blue);

/// @brief Sets pixels of a 3-channel image ROI to a value respective constant value, one for each channel.
/// @ingroup qvipp
/// @see Set(QVImage<uChar, 3> &, uChar, uChar, uChar)
void Set(QVImage<uShort, 3> &img, uShort red, uShort green, uShort blue);

/// @brief Sets pixels of a 3-channel image ROI to a value respective constant value, one for each channel.
/// @ingroup qvipp
/// @see Set(QVImage<uChar, 3> &, uChar, uChar, uChar)
void Set(QVImage<sShort, 3> &img, sShort red, sShort green, sShort blue);

/// @brief Sets pixels of a 3-channel image ROI to a value respective constant value, one for each channel.
/// @ingroup qvipp
/// @see Set(QVImage<uChar, 3> &, uChar, uChar, uChar)
void Set(QVImage<sInt, 3> &img, sInt red, sInt green, sInt blue);

/// @brief Sets pixels of a 3-channel image ROI to a value respective constant value, one for each channel.
/// @ingroup qvipp
/// @see Set(QVImage<uChar, 3> &, uChar, uChar, uChar)
void Set(QVImage<sFloat, 3> &img, sFloat red, sFloat green, sFloat blue);

/// @brief Computes absolute pixel values of a source image and places them into the destination image.
/// @ingroup qvipp
///
/// @param src source image.
/// @param dest destination image.
void Abs(const QVImage<sShort> &src, QVImage<sShort> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes absolute pixel values of a source image and places them into the destination image.
/// @ingroup qvipp
/// @see Not(const QVImage<sShort> &, QVImage<sShort> &)
void Abs(const QVImage<sFloat> &src, QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

// boolean
/// @brief Performs a bit-wise NOT operation on each pixel of an image.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise NOT operation on.
/// @param dest destination image to hold result of bit-wise operation.
void Not(const QVImage<uChar> &src, QVImage<uChar> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise NOT operation on each pixel of an image.
/// @ingroup qvipp
/// @see Not(const QVImage<uChar> &, QVImage<uChar> &)
void Not(const QVImage<uChar,3> &src, QVImage<uChar,3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise inclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise inclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void OrC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise inclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise inclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void OrC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise inclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise inclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void OrC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise inclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise inclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void OrC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise inclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise inclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void OrC(const QVImage<sInt> &src, QVImage<sInt> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise inclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise inclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void OrC(const QVImage<sInt, 3> &src, QVImage<sInt, 3> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise AND operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise AND operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void AndC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise AND operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise AND operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void AndC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise AND operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise AND operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void AndC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise AND operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise AND operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void AndC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise AND operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise AND operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void AndC(const QVImage<sInt> &src, QVImage<sInt> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise AND operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise AND operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void AndC(const QVImage<sInt, 3> &src, QVImage<sInt, 3> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise exclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise exclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void XorC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise exclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise exclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void XorC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise exclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise exclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void XorC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise exclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise exclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void XorC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise exclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise exclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void XorC(const QVImage<sInt> &src, QVImage<sInt> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs a bit-wise exclusive OR operation between each pixel of an image and a constant.
/// @ingroup qvipp
///
/// @param src source image to perform bit-wise exclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void XorC(const QVImage<sInt, 3> &src, QVImage<sInt, 3> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the left.
/// @ingroup qvipp
///
/// @param src source image to perform left shift on.
/// @param dest destination image to hold result of left shift.
/// @param constant value to perform left shift with.
void LShiftC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the left.
/// @ingroup qvipp
///
/// @param src source image to perform left shift on.
/// @param dest destination image to hold result of left shift.
/// @param constant value to perform left shift with.
void LShiftC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the left.
/// @ingroup qvipp
///
/// @param src source image to perform left shift on.
/// @param dest destination image to hold result of left shift.
/// @param constant value to perform left shift with.
void LShiftC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the left.
/// @ingroup qvipp
///
/// @param src source image to perform left shift on.
/// @param dest destination image to hold result of left shift.
/// @param constant value to perform left shift with.
void LShiftC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the left.
/// @ingroup qvipp
///
/// @param src source image to perform left shift on.
/// @param dest destination image to hold result of left shift.
/// @param constant value to perform left shift with.
void LShiftC(const QVImage<sInt> &src, QVImage<sInt> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the left.
/// @ingroup qvipp
///
/// @param src source image to perform left shift on.
/// @param dest destination image to hold result of left shift.
/// @param constant value to perform left shift with.
void LShiftC(const QVImage<sInt, 3> &src, QVImage<sInt, 3> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the natural logarithm of pixel values in a source image and writes the results into
/// the destination image.
/// @ingroup qvipp
///
/// If a source pixel value is zero or negative, the function issues a corresponding warning and continues execution
/// with the corresponding result value.
///
/// @param src source image to perform left shift on.
/// @param dest destination image to hold result of left shift.
void Ln(const QVImage<sFloat> &src, QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the natural logarithm of pixel values in a source image and writes the results into
/// the destination image.
/// @ingroup qvipp
/// @see Ln(const QVImage<sFloat> &, QVImage<sFloat> &)
void Ln(const QVImage<sFloat,3> &src, QVImage<sFloat,3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Squares pixel values of an image and writes them into
/// the destination image.
/// @ingroup qvipp
/// @see Sqr(const QVImage<sFloat> &, QVImage<sFloat> &)
void Sqr(const QVImage<sFloat,1> &src, QVImage<sFloat,1> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the right.
/// @ingroup qvipp
///
/// @param src source image to perform right shift on.
/// @param dest destination image to hold result of right shift.
/// @param constant value to perform right shift with.
void RShiftC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the right.
/// @ingroup qvipp
///
/// @param src source image to perform right shift on.
/// @param dest destination image to hold result of right shift.
/// @param constant value to perform right shift with.
void RShiftC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the right.
/// @ingroup qvipp
///
/// @param src source image to perform right shift on.
/// @param dest destination image to hold result of right shift.
/// @param constant value to perform right shift with.
void RShiftC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the right.
/// @ingroup qvipp
///
/// @param src source image to perform right shift on.
/// @param dest destination image to hold result of right shift.
/// @param constant value to perform right shift with.
void RShiftC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the right.
/// @ingroup qvipp
///
/// @param src source image to perform right shift on.
/// @param dest destination image to hold result of right shift.
/// @param constant value to perform right shift with.
void RShiftC(const QVImage<sInt> &src, QVImage<sInt> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Shifts bits in pixel values to the right.
/// @ingroup qvipp
///
/// @param src source image to perform right shift on.
/// @param dest destination image to hold result of right shift.
/// @param constant value to perform right shift with.
void RShiftC(const QVImage<sInt, 3> &src, QVImage<sInt, 3> &dest, sInt constant, QPoint destROIOffset = QPoint(0,0));

// aritmetic
/// @brief Adds pixel values of two images.
/// @ingroup qvipp
///
/// This function adds corresponding pixel values of two source image buffers and places the
/// results in a destination buffer. Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src1 source image to use pixels as first operands in add.
/// @param src2 source image to use pixels as second operands in add.
/// @param dest destination image to hold result pixels of add.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void Add(const QVImage<uChar> &src1, const QVImage<uChar> &src2, QVImage<uChar> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds pixel values of two images.
/// @ingroup qvipp
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2, QVImage<uChar,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds pixel values of two images.
/// @ingroup qvipp
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<uShort> &src1, const QVImage<uShort> &src2, QVImage<uShort> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds pixel values of two images.
/// @ingroup qvipp
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<uShort,3> &src1, const QVImage<uShort,3> &src2, QVImage<uShort,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds pixel values of two images.
/// @ingroup qvipp
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<sShort> &src1, const QVImage<sShort> &src2, QVImage<sShort> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds pixel values of two images.
/// @ingroup qvipp
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2, QVImage<sShort,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds pixel values of two images.
/// @ingroup qvipp
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2, QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds pixel values of two images.
/// @ingroup qvipp
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2, QVImage<sFloat,3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts pixel values of two images.
/// @ingroup qvipp
///
/// This function substracts corresponding pixel values of two source image buffers and places the
/// results in a destination buffer. Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src1 source image to use pixels as first operands in subtraction.
/// @param src2 source image to use pixels as second operands in subtraction.
/// @param dest destination image to hold result pixels of subtraction.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void Sub(const QVImage<uChar> &src1, const QVImage<uChar> &src2,QVImage<uChar> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts pixel values of two images.
/// @ingroup qvipp
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2,QVImage<uChar,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts pixel values of two images.
/// @ingroup qvipp
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<uShort> &src1, const QVImage<uShort> &src2,QVImage<uShort> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts pixel values of two images.
/// @ingroup qvipp
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<uShort,3> &src1, const QVImage<uShort,3> &src2,QVImage<uShort,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts pixel values of two images.
/// @ingroup qvipp
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<sShort> &src1, const QVImage<sShort> &src2,QVImage<sShort> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts pixel values of two images.
/// @ingroup qvipp
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2,QVImage<sShort,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts pixel values of two images.
/// @ingroup qvipp
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2,QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts pixel values of two images.
/// @ingroup qvipp
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2,QVImage<sFloat,3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images.
/// @ingroup qvipp
///
/// This function multiplies corresponding pixel values of two source image buffers and places the
/// results in a destination buffer. Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src1 source image to use pixels as first operands in multiplication.
/// @param src2 source image to use pixels as second operands in multiplication.
/// @param dest destination image to hold result pixels of multiplication.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void Mul(const QVImage<uChar> &src1, const QVImage<uChar> &src2,QVImage<uChar> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images.
/// @ingroup qvipp
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2,QVImage<uChar,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images.
/// @ingroup qvipp
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<uShort> &src1, const QVImage<uShort> &src2,QVImage<uShort> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images.
/// @ingroup qvipp
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<uShort,3> &src1, const QVImage<uShort,3> &src2,QVImage<uShort,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images.
/// @ingroup qvipp
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<sShort> &src1, const QVImage<sShort> &src2,QVImage<sShort> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images.
/// @ingroup qvipp
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2,QVImage<sShort,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images.
/// @ingroup qvipp
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2,QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images.
/// @ingroup qvipp
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2,QVImage<sFloat,3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by pixel values of another image.
/// @ingroup qvipp
///
/// This function divides corresponding pixel values of two source image buffers and places the
/// results in a destination buffer. Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src1 source image to use pixels as dividends in division.
/// @param src2 source image to use pixels as divisors in division.
/// @param dest destination image to hold result pixels of division.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void Div(const QVImage<uChar> &src1, const QVImage<uChar> &src2,QVImage<uChar> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by pixel values of another image.
/// @ingroup qvipp
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2,QVImage<uChar,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by pixel values of another image.
/// @ingroup qvipp
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<uShort> &src1, const QVImage<uShort> &src2,QVImage<uShort> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by pixel values of another image.
/// @ingroup qvipp
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<uShort,3> &src1, const QVImage<uShort,3> &src2,QVImage<uShort,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by pixel values of another image.
/// @ingroup qvipp
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<sShort> &src1, const QVImage<sShort> &src2,QVImage<sShort> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by pixel values of another image.
/// @ingroup qvipp
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2,QVImage<sShort,3> &dest, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by pixel values of another image.
/// @ingroup qvipp
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2,QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by pixel values of another image.
/// @ingroup qvipp
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2,QVImage<sFloat,3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images and scales the products.
/// @ingroup qvipp
///
/// This function multiplies corresponding pixel values of two input buffers and scales the
/// products using the following formula:
/// \verbatim dest(i,j) = src1(i,j) * src2(i,j) / max_val \endverbatim
///
/// where max_val is the maximum value of the pixel data range.
///
/// @param src1 source image to use pixels as first operands in multiplication.
/// @param src2 source image to use pixels as second operands in multiplication.
/// @param dest destination image to hold result pixels of multiplication.
void MulScale(const QVImage<uChar> &src1, const QVImage<uChar> &src2, QVImage<uChar> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of two images and scales the products.
/// @ingroup qvipp
/// @see MulScale(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &)
void MulScale(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2, QVImage<uChar,3> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Calculates absolute difference between two images.
/// @ingroup qvipp
///
/// This function calculates the absolute pixel-wise difference between two images by the formula:
/// \verbatim dest(x,y) = abs(src1(x,y) - src2(x,y)) \endverbatim
///
/// @param src1 source image to use pixels as first operands in subtraction.
/// @param src2 source image to use pixels as second operands in subtraction.
/// @param dest destination image to hold result pixels of operation.
void AbsDiff(const QVImage<uChar> &src1, const QVImage<uChar> &src2,QVImage<uChar> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Calculates absolute difference between two images.
/// @ingroup qvipp
/// @see AbsDiff(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &)
void AbsDiff(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2,QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

// constant aritmetic
/// @brief Multiplies pixel values of an image by a constant.
/// @ingroup qvipp
///
/// This function multiplies pixel values of an image by a constant value.
/// Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src source image to use pixels as first operands in multiplication.
/// @param dest destination image to hold result pixels of multiplication.
/// @param constant value to use as second operand in multiplication.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void MulC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of an image by a constant.
/// @ingroup qvipp
/// @see MulC(const QVImage<uChar> &, QVImage<uChar> &, uChar constant, int)
void MulC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of an image by a constant.
/// @ingroup qvipp
/// @see MulC(const QVImage<uChar> &, QVImage<uChar> &, uChar constant, int)
void MulC(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Multiplies pixel values of an image by a constant.
/// @ingroup qvipp
/// @see MulC(const QVImage<uChar> &, QVImage<uChar> &, uChar constant, int)
void MulC(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void MulC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void MulC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void MulC(const QVImage<sShort, 3> &src, QVImage<sShort, 3> &dest, sShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void MulC(const QVImage<sFloat, 3> &src, QVImage<sFloat, 3> &dest, sFloat constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
///
/// This function changes the image intensity by adding the value to image pixel values.
/// Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src source image to use pixels as first operands in add.
/// @param dest destination image to hold result pixels of add.
/// @param constant value to use as second operand in add.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void AddC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<sShort, 3> &src, QVImage<sShort, 3> &dest, sShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<sFloat, 3> &src, QVImage<sFloat, 3> &dest, sFloat constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts a constant from pixel values of an image.
/// @ingroup qvipp
///
/// This function changes image intensity by subtracting the constant value from pixel values
/// of an image buffer.
/// Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src source image to use pixels as first operands in subtraction.
/// @param dest destination image to hold result pixels of subtraction.
/// @param constant value to use as second operand in subtraction.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void SubC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts a constant from pixel values of an image.
/// @ingroup qvipp
/// @see void SubC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void SubC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts a constant from pixel values of an image.
/// @ingroup qvipp
/// @see void SubC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void SubC(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Subtracts a constant from pixel values of an image.
/// @ingroup qvipp
/// @see void SubC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void SubC(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void SubC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void SubC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void SubC(const QVImage<sShort, 3> &src, QVImage<sShort, 3> &dest, sShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void SubC(const QVImage<sFloat, 3> &src, QVImage<sFloat, 3> &dest, sFloat constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by a constant.
/// @ingroup qvipp
///
/// This function changes image intensity by dividing pixel values of an image buffer by the
/// constant value.
/// Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src source image to use pixels as first operands in multiplication.
/// @param dest destination image to hold result pixels of multiplication.
/// @param constant value to use as second operand in multiplication.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void DivC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by a constant.
/// @ingroup qvipp
/// @see DivC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void DivC(const QVImage<uShort> &src, QVImage<uShort> &dest, uShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by a constant.
/// @ingroup qvipp
/// @see DivC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void DivC(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Divides pixel values of an image by a constant.
/// @ingroup qvipp
/// @see DivC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void DivC(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void DivC(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest, uChar constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void DivC(const QVImage<uShort, 3> &src, QVImage<uShort, 3> &dest, uShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void DivC(const QVImage<sShort, 3> &src, QVImage<sShort, 3> &dest, sShort constant, int scale = 1, QPoint destROIOffset = QPoint(0,0));

/// @brief Adds a constant to pixel values of an image.
/// @ingroup qvipp
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void DivC(const QVImage<sFloat, 3> &src, QVImage<sFloat, 3> &dest, sFloat constant, QPoint destROIOffset = QPoint(0,0));

// statistics
/// @brief Computes the sum of image pixel values.
/// @ingroup qvipp
///
/// @param src source image to compute pixel sum.
/// @param value return value containing the sum of the values of the pixels of the image.
void Sum(const QVImage<uChar> &src, sDouble & value);

/// @brief Computes the sum of image pixel values.
/// @ingroup qvipp
/// @see Sum(const QVImage<uChar> &, sDouble &)
void Sum(const QVImage<sShort> &src, sDouble & value);

/// @brief Computes the sum of image pixel values, by separate planes.
/// @ingroup qvipp
///
/// @param src source image to compute pixel sum.
/// @param red return value containing the sum of the values of the pixels of the image for the first channel.
/// @param green return value containing the sum of the values of the pixels of the image for the second channel.
/// @param blue return value containing the sum of the values of the pixels of the image for the third channel.
void Sum(const QVImage<uChar,3> &src, sDouble & r, sDouble & g, sDouble & b);

/// @brief Computes the sum of image pixel values, by separate planes.
/// @ingroup qvipp
/// @see Sum(const QVImage<uChar,3> &, sDouble &, sDouble &, sDouble &)
void Sum(const QVImage<sShort,3> &src, sDouble & r, sDouble & g, sDouble & b);

/// @brief Computes the integral image.
/// @ingroup qvipp
///
/// This function transforms a source image Src to the
/// integral image Dst. Pixel values of the destination image Dst are computed using pixel
/// values of the source image Src and the specified value val in accordance with the
/// following formula:
///   \f$ Dst[i,j]=val+ \sum_{k < i} \sum_{l < j} Src[k,l] \f$
/// @param src source image.
/// @param dst integral image.
/// @param val the value to add to dst image pixels (usually 0).

void Integral(const QVImage<uChar,1> &src, QVImage<sInt,1> &dst, sInt val, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the integral image.
/// @ingroup qvipp
///
/// @param src source image.
/// @param dst integral image.
/// @param val the value to add to dst image pixels (usually 0).

void Integral(const QVImage<uChar,1> &src, QVImage<sFloat,1> &dst, sFloat val, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the integral and integral squares images.
/// @ingroup qvipp
///
/// This function transforms an image to integral and integral of pixel squares representations.
/// @param src source image.
/// @param dst1 integral image.
/// @param dst2 integral square image.
/// @param val the value to add to dst1 image pixels (usually 0).
/// @param valSqr the value to add to dst2 image pixels (usually 0).

void SqrIntegral(const QVImage<uChar,1> &src, QVImage<sFloat,1> &dst1, QVImageBuffer<sDouble,1> &dst2, sFloat val, sDouble valSqr, QPoint destROIOffset = QPoint(0,0));


/// @brief Computes the standard deviation of the integral images.
/// @ingroup qvipp
///
/// This function computes std dev image from integral and integral of pixel squares representations,
/// for the desired rect rectangle around each input image pixel.
/// @param src1 integral image.
/// @param src2 integral square image.
/// @param dst std dev (output) image.
/// @param val the value to add to dst1 image pixels (usually 0).
/// @param valSqr the value to add to dst2 image pixels (usually 0).
void RectStdDev(const QVImage<sFloat,1> &src1, const QVImageBuffer<sDouble,1> &src2, QVImage<sFloat,1> &dst, IppiRect rect, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the mean of image pixel values.
/// @ingroup qvipp
///
/// @param src source image to compute pixel mean.
/// @param value return value containing the mean of the values of the pixels of the image.
void Mean(const QVImage<uChar> &src, sDouble & value, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the mean of image pixel values.
/// @ingroup qvipp
/// @see Mean(const QVImage<uChar> &, sDouble &)
void Mean(const QVImage<sShort> &src, sDouble & value, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the mean of image pixel values, by separate planes.
/// @ingroup qvipp
///
/// @param src source image to compute pixel mean.
/// @param red return value containing the mean of the values of the pixels of the image for the first channel.
/// @param green return value containing the mean of the values of the pixels of the image for the second channel.
/// @param blue return value containing the mean of the values of the pixels of the image for the third channel.
void Mean(const QVImage<uChar,3> &src, sDouble & r, sDouble & g, sDouble & b, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the mean of image pixel values, by separate planes.
/// @ingroup qvipp
/// @see Mean(const QVImage<uChar,3> &, sDouble &, sDouble &, sDouble &)
void Mean(const QVImage<sShort,3> &src, sDouble & r, sDouble & g, sDouble & b, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the maximum of image pixel values.
/// @ingroup qvipp
///
/// @param src source image to compute pixel maximum.
/// @param value return value containing the maximum of the values of the pixels of the image.
void Max(const QVImage<uChar> &src, uChar & value);

/// @brief Computes the maximum of image pixel values.
/// @ingroup qvipp
/// @see Max(const QVImage<uChar> &, uChar &)
void Max(const QVImage<sShort> &src, sShort & value);

/// @brief Computes the maximum of image pixel values.
/// @ingroup qvipp
/// @see Max(const QVImage<uChar> &, uChar &)
void Max(const QVImage<sFloat> &src, sFloat & value);

/// @brief Computes the maximum of image pixel values, by separate planes.
/// @ingroup qvipp
///
/// @param src source image to compute pixel maximum.
/// @param red return value containing the maximum of the values of the pixels of the image for the first channel.
/// @param green return value containing the maximum of the values of the pixels of the image for the second channel.
/// @param blue return value containing the maximum of the values of the pixels of the image for the third channel.
void Max(const QVImage<uChar,3> &src, uChar & r, uChar & g, uChar & b);

/// @brief Computes the maximum of image pixel values, by separate planes.
/// @ingroup qvipp
/// @see Max(const QVImage<uChar,3> &, uChar &, uChar &, uChar &)
void Max(const QVImage<sShort,3> &src, sShort &r, sShort &g, sShort &b);

/// @brief Computes the minimum of image pixel values.
/// @ingroup qvipp
///
/// @param src source image to compute pixel minimum.
/// @param value return value containing the minimum of the values of the pixels of the image.
void Min(const QVImage<uChar> &src, uChar & value);

/// @brief Computes the minimum of image pixel values.
/// @ingroup qvipp
/// @see Min(const QVImage<uChar> &, uChar &)
void Min(const QVImage<sShort> &src, sShort & value);

/// @brief Computes the minimum of image pixel values.
/// @ingroup qvipp
/// @see Min(const QVImage<uChar> &, uChar &)
void Min(const QVImage<sFloat> &src, sFloat & value);

/// @brief Computes the minimum of image pixel values, by separate planes.
/// @ingroup qvipp
///
/// @param src source image to compute pixel minimum.
/// @param red return value containing the minimum of the values of the pixels of the image for the first channel.
/// @param green return value containing the minimum of the values of the pixels of the image for the second channel.
/// @param blue return value containing the minimum of the values of the pixels of the image for the third channel.
void Min(const QVImage<uChar,3> &src, uChar & r, uChar & g, uChar & b);

/// @brief Computes the minimum of image pixel values, by separate planes.
/// @ingroup qvipp
/// @see Min(const QVImage<uChar,3> &, uChar &, uChar &, uChar &)
void Min(const QVImage<sShort,3> &src, sShort &r, sShort &g, sShort &b);

// Filters
/// @brief Applies the <i>max</i> filter to an image.
/// @ingroup qvipp
///
/// This function sets each pixel in the destination image to the maximum value of all the
/// source image pixel values in the neighborhood of mask size given.
///
/// This has the effect of increasing the contrast in the image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param colMaskSize mask width.
/// @param rowMaskSize mask height.
void FilterMax(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt colMaskSize, uInt rowMaskSize, QPoint destROIOffset = QPoint(0,0));

/// @brief Applies the <i>max</i> filter to an image.
/// @ingroup qvipp
/// @see FilterMax(const QVImage<sFloat> &, QVImage<sFloat> &, uInt, uInt)
void FilterMax(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt colMaskSize, uInt rowMaskSize, QPoint destROIOffset = QPoint(0,0));

/// @brief Applies the <i>max</i> filter to an image.
/// @ingroup qvipp
/// @see FilterMax(const QVImage<uShort> &, QVImage<uShort> &, uInt, uInt)
void FilterMax(const QVImage<uShort> &src, QVImage<uShort> &dest, uInt colMaskSize, uInt rowMaskSize, QPoint destROIOffset = QPoint(0,0));


/// @brief Applies the <i>min</i> filter to an image.
/// @ingroup qvipp
///
/// This function sets each pixel in the destination image to the minimum value of all the
/// source image pixel values in the neighborhood of mask size given.
///
/// This has the effect of decreasing the contrast in the image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param colMaskSize mask width.
/// @param rowMaskSize mask height.
void FilterMin(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt colMaskSize, uInt rowMaskSize, QPoint destROIOffset = QPoint(0,0));

/// @brief Applies the <i>min</i> filter to an image.
/// @ingroup qvipp
/// @see FilterMin(const QVImage<sFloat> &, QVImage<sFloat> &, uInt, uInt)
void FilterMin(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt colMaskSize, uInt rowMaskSize, QPoint destROIOffset = QPoint(0,0));

/// @brief Applies the <i>min</i> filter to an image.
/// @ingroup qvipp
/// @see FilterMin(const QVImage<sShort> &, QVImage<sShort> &, uInt, uInt)
void FilterMin(const QVImage<sShort> &src, QVImage<sShort> &dest, uInt colMaskSize, uInt rowMaskSize, QPoint destROIOffset = QPoint(0,0));


/// @brief Blurs an image using a simple box filter.
/// @ingroup qvipp
///
/// This function sets each pixel in the output image as the average of all the input image
/// pixels in the rectangular neighborhood of size mask size given.
///
/// This has the effect of smoothing or blurring the input image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param colMaskSize mask width.
/// @param rowMaskSize mask height.
void FilterBox(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt colMaskRadius, uInt rowMaskRadius, QPoint destROIOffset = QPoint(0,0));

/// @brief Blurs an image using a simple box filter.
/// @ingroup qvipp
/// @see FilterBox(const QVImage<uChar> &, QVImage<uChar> &, uInt, uInt)
void FilterBox(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt colMaskRadius, uInt rowMaskRadius, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a Gaussian kernel.
/// @ingroup qvipp
///
/// This function applies a lowpass Gaussian filter to an image ROI.
/// The corresponding kernel is the matrix of either 3x3 or 5x5 size.
/// The 3x3 filter uses the kernel:
///
/// \f$ \left| \begin{array}{ccc} 1/16 & 2/16 & 1/16 \\ 2/16 & 4/16 & 2/16 \\ 1/16 & 2/16 & 1/16 \\ \end{array} \right| \f$
///
/// These filter coefficients correspond to a 2-dimensional Gaussian distribution with standard
/// deviation 0.85.
///
/// The 5x5 filter uses the kernel:
///
/// \f$ \left| \begin{array}{ccccc} 2/571 & 7/571 & 12/571 & 7/571 & 2/571 \\ 7/571 & 31/571 & 52/571 & 31/571 & 7/571 \\ 12/571 & 52/571 & 127/571 & 52/571 & 12/571 \\ 7/571 & 31/571 & 52/571 & 31/571 & 7/571 \\ 2/571 & 7/571 & 12/571 & 7/571 & 2/571 \\ \end{array} \right| \f$
///
/// These filter coefficients correspond to a 2-dimensional Gaussian distribution with standard
/// deviation 1.0.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterGauss(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a Gaussian kernel.
/// @ingroup qvipp
/// @see FilterGauss(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterGauss(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a Gaussian kernel.
/// @ingroup qvipp
/// @see FilterGauss(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterGauss(const QVImage<uChar,3> &src, QVImage<uChar,3> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a Gaussian kernel.
/// @ingroup qvipp
/// @see FilterGauss(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterGauss(const QVImage<sFloat,3> &src, QVImage<sFloat,3> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a general rectangular kernel.
/// @ingroup qvipp
///
/// This function applies a high-pass filter to an image ROI.

/// This function uses the general square kernel defined in the matrix <i>kernel</i> 
/// to filter an image, using a convolution operation.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param kernel matrix containing filter kernel.
void Filter(const QVImage<sFloat> &src, QVImage<sFloat> &dest, const QVMatrix kernel, const QPoint destROIOffset = QPoint(0,0));


/// @brief Filters an image using a spatial kernel that consists of a single column.
/// @ingroup qvipp
///
/// This function uses the vertical column kernel contained in vector <i>kernel</i> to filter an image ROI.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param kernel vector containing the kernel to filter.
void FilterColumn(const QVImage<sFloat> &src, QVImage<sFloat> &dest, const QVVector kernel, const QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a spatial kernel that consists of a single row.
/// @ingroup qvipp
///
/// This function uses the vertical column kernel contained in vector <i>kernel</i> to filter an image ROI.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param kernel vector containing the kernel to filter.
void FilterRow(const QVImage<sFloat> &src, QVImage<sFloat> &dest, const QVVector kernel, const QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a high-pass filter.
/// @ingroup qvipp
///
/// This function applies a high-pass filter to an image ROI.
/// The corresponding kernel is the matrix of either 3x3 or 5x5 size with the following values:
///
/// \f$ \left| \begin{array}{ccc} -1 & -1 & -1 \\ -1 & +8 & -1 \\ -1 & -1 & -1 \\ \end{array} \right| \f$
///
/// for 3x3 size, or
///
/// \f$ \left| \begin{array}{ccccc} -1 & -1 & -1 & -1 & -1 \\ -1 & -1 & -1 & -1 & -1 \\ -1 & -1 & +24 & -1 & -1 \\ -1 & -1 & -1 & -1 & -1 \\ -1 & -1 & -1 & -1 & -1 \\ \end{array} \right| \f$
///
/// For 5x5 size. This filter attenuates low-frequency components and thus sharpens an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterHipass(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a high-pass filter.
/// @ingroup qvipp
/// @see FilterHipass(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterHipass(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a lowpass filter.
/// @ingroup qvipp
///
/// This filter blurs an image by averaging the pixel values over some neighborhood.
/// It works similar to FilterBox function, but uses two fixed mask sizes: 3x3, or 5x5.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterLowpass(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a lowpass filter.
/// @ingroup qvipp
/// @see FilterLowpass(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterLowpass(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a sharpening filter.
/// @ingroup qvipp
///
/// This function applies a sharpening filter to an image ROI.
/// The corresponding kernel is the matrix of 3x3 size with the following values:
///
/// \f$ \left| \begin{array}{ccc} -1/8 & -1/8 & -1/8 \\ -1/8 & 16/8 & -1/8 \\ -1/8 & -1/8 & -1/8 \\ \end{array} \right| \f$
///
/// This filter enhances high-frequency components and thus sharpens an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
void FilterSharpen(const QVImage<uChar> &src, QVImage<uChar> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a sharpening filter.
/// @ingroup qvipp
/// @see FilterSharpen(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSharpen(const QVImage<sFloat> &src, QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a horizontal Sobel kernel.
/// @ingroup qvipp
///
/// This function apply a horizontal Sobel operator to an image ROI.
///
/// This filter has the effect of both enhancing and smoothing horizontal edges of an image.
///
/// \bug this function is not working properly. It will be repaired, for now better use function @ref FilterSobelHorizMask
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
void FilterSobelHoriz(const QVImage<uChar> &src, QVImage<uChar> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a horizontal Sobel kernel.
/// @ingroup qvipp
/// @see FilterSobelHoriz(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSobelHoriz(const QVImage<sShort> &src, QVImage<sShort> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a horizontal Sobel kernel.
/// @ingroup qvipp
/// @see FilterSobelHoriz(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSobelHoriz(const QVImage<sFloat> &src, QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a vertical Sobel kernel.
/// @ingroup qvipp
///
/// This function apply a vertical Sobel operator to an image ROI.
///
/// This filter has the effect of both enhancing and smoothing vertical edges of an image.
///
/// Warning: this function is not working properly. It will be repaired, for now better use function @ref FilterSobelVertMask
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
void FilterSobelVert(const QVImage<uChar> &src, QVImage<uChar> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a vertical Sobel kernel.
/// @ingroup qvipp
/// @see FilterSobelVert(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSobelVert(const QVImage<sShort> &src, QVImage<sShort> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a vertical Sobel kernel.
/// @ingroup qvipp
/// @see FilterSobelVert(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSobelVert(const QVImage<sFloat> &src, QVImage<sFloat> &dest, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a horizontal Sobel kernel.
/// @ingroup qvipp
///
/// This function apply a horizontal Sobel operator to an image ROI.
///
/// The appropriate kernel is the matrix of 3x3 size, or either 3x3 or 5x5 size
/// in accordance with mask parameter of the corresponding function flavors.
/// The kernels have the following values:
///
/// \f$ \left| \begin{array}{ccc} 1 & 2 & 1 \\ 0 & 0 & 0 \\ -1 & -2 & -1 \\ \end{array} \right| \f$
///
/// for 3x3 size, or
///
/// \f$ \left| \begin{array}{ccccc} 1 & 4 & 6 & 4 & 1 \\ 2 & 8 & 12 & 8 & 2 \\ 0 & 0 & 0 & 0 & 0 \\ -2 & -8 & -12 & -8 & -2 \\ -1 & -4 & -6 & -4 & -1 \\ \end{array} \right| \f$
///
/// For 5x5 size. This filter has the effect of both enhancing and smoothing horizontal edges of an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterSobelHorizMask(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a vertical Sobel kernel.
/// @ingroup qvipp
///
/// This function apply a vertical Sobel operator to an image ROI. The appropriate kernel is the
/// matrix of 3x3 size, or either 3x3 or 5x5 size in accordance with mask parameter of the
/// corresponding function flavors. The kernels have the following values:
///
/// \f$ \left| \begin{array}{ccc} -1 & 0 & 1 \\ -2 & 0 & 2 \\ -1 & 0 & 1 \\ \end{array} \right| \f$
///
/// for 3x3 size, or
///
/// \f$ \left| \begin{array}{ccccc} -1 & -2 & 0 & 2 & 1 \\ -4 & -8 & 0 & 8 & 4 \\ -6 & -12 & 0 & 12 & 6 \\ -4 & -8 & 0 & 8 & 4 \\ -1 & -2 & 0 & 2 & 1 \\ \end{array} \right| \f$
///
/// For 5x5 size. This filter has the effect of both enhancing and smoothing vertical edges of an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterSobelVertMask(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Applies horizontal Sobel filter with border.
/// @ingroup qvipp
///
/// This function is similar to FilterSobelHorizMask, but automatically creates a required border and
/// define appropriate pixel values.
///
/// Function FilterSobelHorizGetBufferSize should be used to initialize sizes for a buffer QVImage<uChar>
/// image, which will hold temporary data for the function operations.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param buffer buffer image to hold temporal data during function operation.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
/// @see FilterSobelHorizGetBufferSize
void FilterSobelHorizBorder(const QVImage<uChar> &src, QVImage<sShort> &dest, QVImage<uChar> &buffer, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Applies vertical Sobel filter with border.
/// @ingroup qvipp
///
/// This function is similar to FilterSobelVertMask, but uses negative values for the mask applied,
/// and automatically create a required border and define appropriate pixel values.
///
/// Function FilterSobelNegVertGetBufferSize should be used to initialize sizes for a buffer QVImage<uChar>
/// image, which will hold temporary data for the function operations.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param buffer buffer image to hold temporal data during function operation.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
/// @see FilterSobelNegVertGetBufferSize
void FilterSobelNegVertBorder(const QVImage<uChar> &src, QVImage<sShort> &dest, QVImage<uChar> &buffer, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a median filter.
/// @ingroup qvipp
///
/// This function sets each pixel in the output buffer as the median value of all the input pixel
/// values taken in the neighborhood of the processed pixel.
///
/// This filter removes noise and does not cut out signal brightness drops, as an averaging filter does.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param colMaskSize mask width.
/// @param rowMaskSize mask height.
void FilterMedian(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt colMaskRadius = 3, uInt rowMaskRadius = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a median filter.
/// @ingroup qvipp
/// @see FilterMedian(const QVImage<uChar> &, QVImage<uChar> &, uInt, uInt)
void FilterMedian(const QVImage<sShort> &src, QVImage<sShort> &dest, uInt colMaskRadius = 3, uInt rowMaskRadius = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Changes an image size.
/// @ingroup qvipp
///
/// This function copies the content of the ROI in the source image, to the ROI of the
/// destination image, adjusting it's original size to make it fit in the destination size
/// It has the effect of resizing the original image, to fit the destination image, if both ROI's
/// hold the whole of their respective images.
///
///	- IPPI_INTER_NN. Nearest neighbor interpolation
///	- IPPI_INTER_LINEAR. Linear interpolation
///	- IPPI_INTER_CUBIC. Cubic interpolation
///	- IPPI_INTER_SUPER. Supersampling interpolation, cannot be applied for image enlarging
///	- IPPI_INTER_LANCZOS. Interpolation with Lanczos window.
void Resize(const QVImage<uChar> &src, QVImage<uChar> &dest, int interpolation = IPPI_INTER_LINEAR);

/// @brief Changes an image size.
/// @ingroup qvipp
/// @see Resize(const QVImage<uChar> &, QVImage<uChar> &, int)
void Resize(const QVImage<sFloat> &src, QVImage<sFloat> &dest, int interpolation = IPPI_INTER_LINEAR);

/// @brief Changes an image size.
/// @ingroup qvipp
/// @see Resize(const QVImage<uShort> &, QVImage<uShort> &, int)
void Resize(const QVImage<uShort> &src, QVImage<uShort> &dest, int interpolation = IPPI_INTER_LINEAR);


/// @brief Changes an image size.
/// @ingroup qvipp
/// @see Resize(const QVImage<uChar> &, QVImage<uChar> &, int)
void Resize(const QVImage<uChar,3> &src, QVImage<uChar,3> &dest, int interpolation = IPPI_INTER_LINEAR);

/// @brief Allocates size in a buffer image for temporary data for function Canny.
/// @ingroup qvipp
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// Canny. Initializes dimensions for that buffer image, appropriated for canny algorithm.
///
/// @param src source image to apply later Canny operator.
/// @param buffer buffer image to initialize for further operations.
/// @see Canny
void CannyGetSize(const QVGenericImage &src, QVImage<uChar> &buffer);

/// @brief Allocates size in a buffer image for temporary data for function FilterSobelHoriz.
/// @ingroup qvipp
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// FilterSobelHoriz. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param src source image to apply later filter.
/// @param buffer buffer image to initialize for further operations.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
/// @see FilterSobelHoriz
void FilterSobelHorizGetBufferSize(const QVGenericImage &src, QVImage<uChar> &buffer, uInt maskSize = 3);

/// @brief Allocates size in a buffer image for temporary data for function FilterSobelNegVert.
/// @ingroup qvipp
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// FilterSobelNegVert. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param src source image to apply later filter.
/// @param buffer buffer image to initialize for further operations.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
/// @see FilterSobelNegVert
void FilterSobelNegVertGetBufferSize(const QVGenericImage &src, QVImage<uChar> &buffer, uInt maskSize = 3);

/// @brief Allocates size in a buffer image for temporary data for function MinEigenValGet.
/// @ingroup qvipp
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// MinEigenVal. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param src source image to apply later MinEigenVal function.
/// @param buffer buffer image to initialize for further operations.
/// @param apertureSize Size (in pixels) of the derivative operator used by the function, possible values are 3 or 5.
/// @param avgWindow Size of the blurring window in pixels, possible values are 3 or 5.
/// @see MinEigenVal
void MinEigenValGetBufferSize( const QVGenericImage &src, QVImage<uChar> &buffer, uInt apertureSize = 5, uInt avgWindow = 5);

/// @brief Allocates size in a buffer image for temporary data for function family FloodFill.
/// @ingroup qvipp
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// family FloodFill. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param src source image to apply later flood fill function.
/// @param buffer buffer image to initialize for further operations.
/// @see FloodFillRange
void FloodFillGetSize(const QVGenericImage &src, QVImage<uChar> &buffer);

/// @brief Performs Canny algorithm for edge detection.
/// @ingroup qvipp
///
/// This function finds edges in the source image ROI and stores them into the output image
/// dest using the Canny algorithm.
///
/// The function requires a temporary buffer image, that should have been initialized with CannyGetSize function.
///
/// Images dX and dY should be filtered images from the original, from which we desire to obtain the contours, using
/// horizontal and vertical Sobel filters.
///
/// A code example of a full canny extractor, given a source image, is described in the code below:
/// \code
///	[...]
///
///	QVImage<sFloat>	imageFloat(cols, rows), dX(cols, rows), dY(cols, rows, QPoint destROIOffset = QPoint(0,0));
///	QVImage<uChar>	canny(cols, rows), buffer;
///
///	Convert(image, imageFloat, QPoint destROIOffset = QPoint(0,0));
///	FilterSobelHorizMask(imageFloat,dY,3);
///	FilterSobelVertMask(imageFloat,dX,3);
///	CannyGetSize(canny, buffer);
///	Canny(dX, dY, canny, buffer);
///
///	[...]
/// \endcode
///
/// @param dX source image x-derivative.
/// @param dY source image y-derivative.
/// @param dest output image of the detected edges.
/// @param buffer image to hold temporary data during function operations.
/// @param low lower threshold for edges detection.
/// @param high upper threshold for edges detection.
/// @see CannyGetSize
/// @see FilterSobelHoriz
/// @see FilterSobelVert
/// @see FilterSobelHorizBorder
/// @see FilterSobelNegVertBorder
void Canny(QVImage<sShort> &dX, QVImage<sShort> &dY, QVImage<uChar> &dest,
		QVImage<uChar> &buffer, double low = 50.0, double high = 150.0);

/// @brief Performs Canny algorithm for edge detection.
/// @ingroup qvipp
/// @see Canny(QVImage<sShort> &, QVImage<sShort> &, QVImage<uChar> &, QVImage<uChar> &, double, double)
void Canny(QVImage<sFloat> &dX, QVImage<sFloat> &dY, QVImage<uChar> &dest,
		QVImage<uChar> &buffer, double low = 50.0, double high = 150.0, QPoint destROIOffset = QPoint(0,0));

/// @brief Calculates the minimal eigen value of image blocks for corner detection.
/// @ingroup qvipp
///
/// This function takes a block around the pixel and computes the first derivatives Dx and Dy.
///
/// This operation is performed for every pixel of the image using either Sobel or Scharr kernel
/// in accordance with the kernType parameter.
///
/// The size of the Sobel kernel may be specified the parameter apertureSize.
/// If this parameter is set to 3 - the function used 3x3 kernel, if it set to 5 - the function uses 5x5 kernel.
///
/// Only 3x3 size is available for the Scharr kernel, therefore the parameter apertureSize must be set to
/// 3 if the Scharr kernel is used.
///
/// @param src source image.
/// @param dest destination image to hold the minimal eigen values.
/// @param buffer image to hold temporary data during function operations.
/// @param apertureSize Size (in pixels) of the derivative operator used by the function, possible values are 3 or 5.
/// @param avgWindow Size of the blurring window in pixels, possible values are 3 (3x3 mask size) or 5 (5x5 mask size).
/// @param kernel Specifies the type of kernel used to compute derivatives, possible values are:
///	- ippKernelSobel. Values for aperture size can be 3, or 5.
///	- ippKernelScharr. Value for aperture size can only be 3.
/// @see MinEigenValGetBufferSize
void MinEigenVal(const QVImage<uChar> &src, QVImage<sFloat> &dest, QVImage<uChar> &buffer,
	uInt apertureSize = 3, uInt avgWindow = 5, IppiKernelType kernel = ippKernelSobel, QPoint destROIOffset = QPoint(0,0));

/// @brief Allocates size in a buffer image for temporary data for function FastMarching.
/// @ingroup qvipp
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// FastMarching. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param image source image to apply Fast Marching algorithm.
/// @param buffer buffer image to initialize for further operations.
/// @see FastMarching
void FastMarchingGetBufferSize(const QVGenericImage &image, QVImage<uChar> &buffer);

/// @brief Calculates distance transform to closest zero pixel for all non-zero pixels of source image using fast marching method.
/// @ingroup qvipp
///
/// This function computes the distance from the closest zero pixel to each image pixel according to the Fast Marching
/// Method (FMM). For further info, see:
///
/// A.Telea. An Image Inprinting Technique Based on the Fast Marching Method. Journal of Graphic Tools, vol.9, No.1.
/// ACM Press, 2004.
///
/// @param mask source image to obtain distances to closest zero pixel.
/// @param distances image to hold distances to closest zero pixel.
/// @param radius Radius of the neighborhood of the marked area.
/// @param buffer image to hold temporary data during function operations.
/// @see FastMarchingGetBufferSize
void FastMarching(const QVImage<uChar> &mask, QVImage<sFloat> &distances, int radius, QVImage<uChar> &buffer, QPoint destROIOffset = QPoint(0,0));

/// @brief Restores unknown image pixels.
/// @ingroup qvipp
///
/// This function reconstructs damaged part of the image, or removes a selected object.
/// The image part to restore is defined by the mask image.
/// Different distant transforms can be used, but the Fast Marching Method (implemented by
/// FastMarching function) provides the best results.
/// The order of pixel restoration is defined by the distance through the initialization the
/// inpainting structure pState by the function ippiInpaintInitAlloc.
/// Pixels are restored in according to the growing of their distance value.
/// When a pixel is inpainted, it is treated as the known one.
/// Two algorithms of direct inpainting are supported
///	- image restoration of the unknown pixel by the weighted sum of approximations by known pixels in the neighborhood.
///	- image restoration based on the Navier-Stokes equations.
///
/// A code example to show the use of this function is below:
/// \code
/// [...]
/// 
/// QVImage<uChar,3> image;	// image to inpaint
/// QVImage<uChar> mask;		// mask, indicating unknown pixels for image (0 known, >0 unknown)
/// 
/// [...]
/// 
/// QVImage<uChar> buffer;		// temporary buffer for FastMarching function
/// QVImage<sFloat> distances(image.getCols(), image.getRows());	// image to hold distances to zero pixels
/// QVImage<uChar,3> inpaint(image.getCols(),image.getRows());	// inpainted image from original
/// 
/// FastMarchingGetBufferSize(mask, buffer);		// initiating buffer image for FastMarching function
/// FastMarching(mask, distances, radius, buffer);		// calculating distances to zero pixels for image
/// Inpaint(image, mask, distances, inpaint, radius);	// inpainting
/// 
/// [...]
/// \endcode
///
/// @param src source image to restore.
/// @param mask mask image. Values different from zero indicate unknown pixels.
/// @param distances image to hold distances to closest zero pixel. Must have been pre-calculated.
/// @param dest destination image to hold restored image.
/// @param radius Radius of the neighborhood of the marked area, used for inpainting.
/// @param flag indicates the algorithm for direct inpainting:
///		- IPP_INPAINT_TELEA. Use weighted sum of approximations by known pixels in the neighborhood.
///		- IPP_INPAINT_NS. Use Navier-Stokes equations.
void Inpaint(const QVImage<uChar> &src, const QVImage<uChar> &mask, const QVImage<sFloat> &distances,
		QVImage<uChar> &dest, int radius = 4,  IppiInpaintFlag flag = IPP_INPAINT_TELEA, QPoint destROIOffset = QPoint(0,0));

/// @brief Overload for function Inpaint, for different channel number images.
/// @ingroup qvipp
/// @see Inpaint(const QVImage<uChar> &, const QVImage<uChar> &, const QVImage<sFloat> &, QVImage<uChar> &, int, IppiInpaintFlag)
void Inpaint(const QVImage<uChar,3> &src, const QVImage<uChar> &mask, const QVImage<sFloat> &distances,
		QVImage<uChar,3> &dest, int radius = 4,  IppiInpaintFlag flag = IPP_INPAINT_TELEA, QPoint destROIOffset = QPoint(0,0));

/// @brief Calculates frequency histogram for a given set of values, for a QVImage
/// @ingroup qvipp
///
/// This function computes the intensity histogram of an image in the ranges specified by the QList 'values'.
/// Default value for this QList is empty list, indicating the set of integer which cover the group of valid values for
/// uChar type (from 0 to 255).
///
/// \todo fill pLevels array with values from QList 'values'
///
/// @param src source image to obtain histogram values.
/// @param values list of QVImage data type values, to obtain frequency in histogram.
/// @return  a QMap variable, that maps pixel values, with their frequency of depicting in the image.
QMap<sInt, int> HistogramRange(const QVImage<uChar, 1> &src, QList<uChar> values);

/// @brief Calculates frequency histogram for a given set of values, for a QVImage
/// @ingroup qvipp
///
/// This function computes the intensity histogram of an image in the ranges from 0 to 255.
///
/// @param src source image to obtain histogram values.
/// @returns a QVector list of 256 integer values, that maps pixel values, with their frequency of apperance
/// in the image.
QVector<int> HistogramRange(const QVImage<uChar, 1> &src);

/// @brief Performs flood filling of pixels with values in the specified range in the connected area on an image.
/// @ingroup qvipp
///
/// This function performs flood filling of the group of connected pixels in the seed pixel
/// neighborhoods whose pixel values \f$ v_i \f$ satisfies the following condition:
///
/// \f$ v_i - minDelta \leq v_0 \leq v_i + maxDelta \f$
/// 
/// where \f$ v_0 \f$ is the pixel value of the seed point. Values of these pixel is set to the <i>newVal</i> value.
///
/// The function check 4-connected neighborhood of each pixel, i.e., side neighbors.
///
/// The function requires a temporary buffer whose size should be computed with the function
/// @ref FloodFillGetSize beforehand.
///
/// @param img image to flood fill.
/// @param buffer temporary buffer image previously initialized with @ref FloodFillGetSize.
/// @param seedX column coordinate for the seed.
/// @param seedY row coordinate for the seed.
/// @param newVal Value to fill with for one-channel data.
/// @param minDelta Minimum difference between neighbor pixels for one-channel data.
/// @param maxDelta Maximum difference between neighbor pixels for one-channel data.
/// @see FloodFillGetSize
void FloodFillRange4Connected(QVImage<uChar> &img, QVImage<uChar> &buffer,
	uInt seedX, uInt seedY, uChar newVal, uChar minDelta, uChar maxDelta);

/// @brief Performs gradient flood filling of connected area on an image.
/// @ingroup qvipp
/// This function performs flood filling of the group of connected pixels in the seed pixel
/// neighborhoods whose pixel values v satisfies the following conditions:
///
/// \f$ v_0 – d_{lw} ≤ v ≤ v_0 + d_{up} \f$
///
/// where \f$ v_0 \f$ is the value of at least one of the current pixel neighbors, which already belongs
/// to the refilled area, and \f$ d_{lw} \f$, dup are minDelta, maxDelta, respectively. Values of these
/// pixel is set to the newVal value.
///
/// The function check 4-connected neighborhood of each pixel, i.e., side neighbors.
///
/// The function requires a temporary buffer whose size should be computed with the function
/// @ref FloodFillGetSize beforehand.
///
/// @param img image to flood fill.
/// @param buffer temporary buffer image previously initialized with @ref FloodFillGetSize.
/// @param seedX column coordinate for the seed.
/// @param seedY row coordinate for the seed.
/// @param newVal Value to fill with for one-channel data.
/// @param minDelta Minimum difference between neighbor pixels for one-channel data.
/// @param maxDelta Maximum difference between neighbor pixels for one-channel data.
/// @see FloodFillGetSize
void FloodFillGradient4Connected(QVImage<uChar> &img, QVImage<uChar> &buffer,
	uInt seedX, uInt seedY, uChar newVal, uChar minDelta, uChar maxDelta);

/// @brief Applies Gauss filter with border.
/// @ingroup qvipp
///
/// This function applies the Gaussian filter to the source image ROI pSrc. The kernel of this filter
/// is the matrix of size \f$ kernelSize x kernelSize \f$ with the standard deviation \f$ sigma \f$. The values of the
/// elements of the Gaussian kernel are calculated according to the formula:
///
/// \f[ G(i,j) = e^{\displaystyle-\frac{\displaystyle(K/2-i)^2 + (K/2-j)^2}{\displaystyle2\sigma^2}} \f]
///
/// and then are normalized.
/// The function requires a working image buffer whose size should be computed by the function @ref FilterGaussGetBufferSize
/// beforehand.
/// @param src source image to apply filter
/// @param dest resulting image.
/// @param buffer buffer image initiated beforehand by the function @ref FilterGaussGetBufferSize.
/// @param kernelSize specifies the size of the Gaussian kernel, must be odd and greater than or equal to 3.
/// @param sigma standard deviation of the Gaussian kernel.
/// @param borderType type of border. Following values are possible:
///	- ippBorderConst. Values of all border pixels are set to constant.
///	- ippBorderRepl. Replicated border is used.
///	- ippBorderWrap. Wrapped border is used
///	- ippBorderMirror. Mirrored border is used
///	- ippBorderMirrorR. Mirrored border with replication is used
/// @param borderValue The constant value to assign to the pixels in the constant border (not applicable for other border’s type).
/// @see FilterGaussGetBufferSize
void FilterGaussBorder(const QVImage<sFloat, 1> &src, QVImage<sFloat, 1> &dest, QVImage<uChar> &buffer, sInt kernelSize, sFloat sigma, IppiBorderType borderType = ippBorderConst, sFloat borderValue = 0.0, QPoint destROIOffset = QPoint(0,0));

/// @brief Computes the size of the external buffer for the Gaussian filter with border.
/// @ingroup qvipp
///
/// This function computes the size of the external buffer that is required for the filter function
/// FilterGaussBorder. This buffer can be used to filter an image whose width and height are equal
/// to or less than param QVGenericImage image, and the kernel size is equal to or less than kernelSize.
///
/// @param src source image to apply later filter.
/// @param buffer buffer image to initialize for further operations.
/// @param kernelSize Size of the Gaussian kernel, odd, greater than or equal to 3.
/// @see FilterGaussBorder
void FilterGaussGetBufferSize(const QVGenericImage &image, QVImage<uChar> &buffer, uInt kernelSize);

/// @brief Finds coordinates of peaks (maximums or minimums) with absolute value exceeding threshold value.
/// @ingroup qvipp
///
/// This function detects local maximum pixels in the source image, and stores their coordinates in
/// the <i>points</i> point list.
/// The neighborhood O(i, j) for the search is defined by the parameter norm. The
/// number of detected maximums is returned in pPeakCount[0]. The operation is stopped
/// The function requires a working image buffer whose size should be computed by the function
/// @ref FindPeaks3x3GetBufferSize beforehand.
///
/// @param img image to obtain from peaks.
/// @param buffer buffer image initiated beforehand by the function @ref FindPeaks3x3GetBufferSize.
/// @param points QPoint list to store the resulting point list.
/// @param threshold threshold value.
/// @param norm Specifies type of the norm to form the mask for maximum search:
///	- ippiNormInf. Infinity norm (8-connectivity, 3x3 rectangular mask);
///	- ippiNormL1. L1 norm (4-connectivity, 3x3 cross mask).
/// @param border Border value, only pixel with distance from the edge of the image greater than border are processed.
/// @param maxPeakCount Maximum number of peaks.
/// @see FindPeaks3x3GetBufferSize
void FindPeaks3x3(const QVImage<sInt, 1> &img, QVImage<uChar> &buffer, QList<QPoint> &points, sInt threshold, IppiNorm norm, uInt border, uInt maxPeakCount);

/// @brief Finds coordinates of peaks (maximums or minimums) with absolute value exceeding threshold value.
/// @ingroup qvipp
/// @see FindPeaks3x3(const QVImage<sInt, 1> &, QVImage<uChar> &, QList<QPoint> &, sInt, IppiNorm, uInt, uInt)
void FindPeaks3x3(const QVImage<sFloat, 1> &img, QVImage<uChar> &buffer, QList<QPoint> &points, sFloat threshold, IppiNorm norm, uInt border, uInt maxPeakCount);


/// @brief Computes the size of the working buffer for the peak search.
/// @ingroup qvipp
///
/// This function computes the size of the working image buffer required for the function
/// FindPeaks3x3. The image buffer can be used to filter images with ROI width that is less
/// than or equal to the image ROI width.
///
/// @param image source image to apply later filter.
/// @param buffer buffer image to initialize for further operations.
/// @see FindPeaks3x3(const QVImage<sInt, 1> &, QVImage<uChar> &, sInt, IppiNorm, uInt, QList<QPoint> &)
void FindPeaks3x3GetBufferSize(const QVImage<sInt,1> &image, QVImage<uChar> &buffer);

/// @brief Computes the size of the working buffer for the peak search.
/// @ingroup qvipp
/// @see FindPeaks3x3GetBufferSize(const QVImage<sInt,1> &, QVImage<uChar> &)
/// @see FindPeaks3x3
void FindPeaks3x3GetBufferSize(const QVImage<sFloat,1> &image, QVImage<uChar> &buffer);

/// @brief Filters an image using a Laplacian kernel.
/// @ingroup qvipp
///
/// This function applies a highpass Laplacian filter to an image ROI. The corresponding kernel
/// is the matrix of either 3x3 or 5x5 size with the following values:
/// \f$ \left| \begin{array}{ccc} -1 & -1 & -1 \\ -1 & 8 & -1 \\ -1 & -1 & -1 \\ \end{array} \right| \f$
///
/// for 3x3 size, or
///
/// \f$ \left| \begin{array}{ccccc} -1 & -3 & -4 & -3 & -1 \\ -3 & 0 & 6 & 0 & -3 \\ -4 & 6 & 20 & 6 & -4 \\ -3 & 0 & 6 & 0 & -3 \\ -1 & -3 & -4 & -3 & -1 \\ \end{array} \right| \f$
///
/// For 5x5 size. This filter helps locate zero crossings in an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterLaplace(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a Laplacian kernel.
/// @ingroup qvipp
/// @see FilterLaplace(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterLaplace(const QVImage<sShort> &src, QVImage<sShort> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Filters an image using a Laplacian kernel.
/// @ingroup qvipp
/// @see FilterLaplace(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterLaplace(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs perspective warping of the source image using the given transform coefficients.
/// @ingroup qvipp
///
/// @todo Document this function
///
/// @param src source image to wrap.
/// @param dest destination image to contain wrapped image.
/// @param rectifyingHomography homography matrix corresponding to the wrapping.
/// @param interpolation interpolation type. Selects the algorithm for interpolate pixel values. Possible values for this parameter are:
///	- IPPI_INTER_NN. Nearest neighbor interpolation
///	- IPPI_INTER_LINEAR. Linear interpolation
///	- IPPI_INTER_CUBIC. Cubic interpolation
void	WarpPerspective(const QVImage<uChar> &src, QVImage<uChar> &dest, const QVMatrix &rectifyingHomography, int interpolation = IPPI_INTER_LINEAR, QPoint destROIOffset = QPoint(0,0));

/// @brief Performs perspective warping of the source image using the given transform coefficients.
/// @ingroup qvipp
/// @see WarpPerspective(const QVImage<uChar> &, QVImage<uChar> &, const QVMatrix &, int, QPoint)
void	WarpPerspective(const QVImage<uChar,3> &src, QVImage<uChar,3> &dest, const QVMatrix &rectifyingHomography, int interpolation = IPPI_INTER_LINEAR, QPoint destROIOffset = QPoint(0,0));

#endif
