/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvpolylinef.cpp
/// @brief File from the QVision library. Contains operations on polylines.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvcore/qvdefines.h>
#include <qvip/qvpolylinef.h>
#include <qvip/qvpolyline.h>
#include <qvmath/qvmatrixalgebra.h>

#include <iostream>
#include <float.h>

//////////////////////////////////////////////
// QVPolylineFs

// Iterative point elimination:
#ifndef DOXYGEN_IGNORE_THIS
class ClassAuxIPE_F
	{
	public:
		double cost;
		int index;
		ClassAuxIPE_F *prev,*next;
	};

bool costLessThan(ClassAuxIPE_F* &s1,ClassAuxIPE_F* &s2)
	{
	return s1->cost < s2->cost;
	}

bool indexLessThan(ClassAuxIPE_F* &s1,ClassAuxIPE_F* &s2)
	{
	return s1->index < s2->index;
	}

inline double costElimination(const QVPolylineF &polyline,int ia, int ib, int ic)
	{
	double   xA,yA,xB,yB,xC,yC;
	xA = polyline[ia].x(); yA=polyline[ia].y();
	xB = polyline[ib].x(); yB=polyline[ib].y();
	xC = polyline[ic].x(); yC=polyline[ic].y();
	if((xA != xC) or (yA != yC))
		return ABS(xA*(yC-yB) + xB*(yA-yC) + xC*(yB-yA)) / sqrt((xA-xC)*(xA-xC)+(yA-yC)*(yA-yC));
	else
		return sqrt((xB-xC)*(xB-xC)+(yB-yC)*(yB-yC));
	}

class auxLine_F {
	public:
	auxLine_F(double l1,double l2,double l3,bool ok) : l1(l1),l2(l2),l3(l3),ok(ok) {};
	double l1,l2,l3;
	bool ok;
};
#endif

double IterativePointElimination(const QVPolylineF &polyline, QVPolylineF &result,
	const double param, bool maxNumberOfPointsMethod,bool intersectLines,
	double *max_removed_cost)
	{
	const uInt tot_siz = polyline.size();
	QList<ClassAuxIPE_F*> list;

	// We start with an empty list:
	result.clear();

	// Maximum removed cost initialized to zero:
	if(max_removed_cost != NULL) *max_removed_cost = 0.0;

	// Only for polylines with 3 points or more; otherwise, the same
	// input polyline is returned:
	if(polyline.size()<3)
		{
		result = polyline;
		return FLT_MAX;
		}

	// Initialization of main data structure:
	for(uInt i=0;i<tot_siz;i++)
		list.push_back(new ClassAuxIPE_F);

	for(uInt i=0;i<tot_siz;i++)
		{
		int ia = (i==0)?tot_siz-1:i-1, ib = i, ic = (i==tot_siz-1)?0:i+1;
		list[i]->cost = costElimination(polyline,ia,ib,ic);
		list[i]->index = ib;
		list[i]->prev = list[ia];
		list[i]->next = list[ic];
		}
	if(not polyline.closed) // If not closed, never eliminate end points:
		{
		list[0]->cost = FLT_MAX;
		list[tot_siz-1]->cost = FLT_MAX;
		}
	qSort(list.begin(),list.end(),costLessThan);

	// Main loop:
	while(TRUE)
		{
		// Stop condition:
		if( (list.size() == 3) or // Minimal size of a polyline.
		    ((not maxNumberOfPointsMethod) and (list[0]->cost > param)) or
		    ((maxNumberOfPointsMethod) and
		     (list.size() <= static_cast<int>(param))) )
			break;

		// Removal of best point (first in the list): 
		ClassAuxIPE_F *elem = list.takeAt(0),
			    *elemPrev = list.takeAt(list.indexOf(elem->prev)),
			    *elemNext = list.takeAt(list.indexOf(elem->next));
		elemPrev->next = elem->next;
		elemNext->prev = elem->prev;
		if(elemPrev->cost != FLT_MAX)
			elemPrev->cost = costElimination(polyline,elemPrev->prev->index,
								elemPrev->index,
								elemPrev->next->index);
		if(elemNext->cost != FLT_MAX)
		elemNext->cost = costElimination(polyline,elemNext->prev->index,
							elemNext->index,
							elemNext->next->index);

		// Binary (fast) insertion of neighbours in data structure:
		int here;
		for(int i=0;i<2;i++)
			{
			ClassAuxIPE_F* newelem = ((i==0)?elemNext:elemPrev);
			int first=0,last=list.size()-1;
			while (first <= last) {
				int mid = (first + last) / 2;
				if (newelem->cost > list[mid]->cost)
					first = mid + 1;
				else if (newelem->cost < list[mid]->cost)
					last = mid - 1;
				else
					{
					here = mid;
					break;
					}
			}
			if(first>last)
				here=first;
			list.insert(here,newelem);

			}

		if(max_removed_cost != NULL)
			if(elem->cost > *max_removed_cost)
				*max_removed_cost = elem->cost;
		delete elem;
		}

	// We will return the cost of the first non deleted point:
	double return_value = list.first()->cost;

	// Once IPE finished, sort the list by position in original polyline:
	qSort(list.begin(),list.end(),indexLessThan);

	// Now, postprocess, fitting lines:
	QList<ClassAuxIPE_F*>::iterator it = list.begin();
	if(intersectLines)
		{
		// Line intersection computation (could be subpixel, in fact...):
		double ratio_eig=1.0;
		QList<auxLine_F> lines;
		for(int i=0;i<list.size();i++)
			{
 			// If not closed, do not need to compute last line:
			if((not polyline.closed) and (i==list.size()-1))
				break;
			int i1 = list[i]->index;
			int i2 = list[(i+1)%list.size()]->index;
			if(i2<i1) i2 += tot_siz;
			int dist = i2-i1+1;
			#define MIN_PIXELS_IPE_LINE 15
			if(dist >= MIN_PIXELS_IPE_LINE)
				{
				i1 = (i1+dist/5)%tot_siz;
				i2 = (i2-dist/5)%tot_siz;
				dist = dist-2*(dist/5);
				}
			else 
				{
				dist = i2-i1+1;
				i1 = i1%tot_siz;
				i2 = i2%tot_siz;
				}
	
			double x=0,y=0,xx=0,xy=0,yy=0;
			uInt j=i1;
			do
				{
				x += polyline[j].x();
				y += polyline[j].y();
				xx += polyline[j].x()*polyline[j].x();
				xy += polyline[j].x()*polyline[j].y();
				yy += polyline[j].y()*polyline[j].y();
				j = (j+1)%tot_siz;
				} while(j!=(i2+1)%tot_siz);
			double l1,l2,l3;
			x /= dist; y /= dist; xx /= dist; xy /= dist; yy /= dist;
			// If line does not fit well, just put old point instead of intersection:
			ratio_eig = homogLineFromMoments(x,y,xx,xy,yy,l1,l2,l3);
			lines.push_back(auxLine_F(l1,l2,l3,ratio_eig < 0.1));
			}

		for(int i=0;i<list.size();i++)
			{
			QPointF oldPoint = polyline[list[i]->index];
			if( (not polyline.closed) and ((i==0) or (i==list.size()-1)))
				{
				// If not closed, just include end points:
				result.append(oldPoint);
				continue;
				}
			int ant = (i-1+list.size())%list.size();
			int post = (i+1)%list.size();
			double 	newx = (lines[i].l2)*(lines[ant].l3) - (lines[i].l3)*(lines[ant].l2);
			double	newy = -(lines[i].l1)*(lines[ant].l3) + (lines[i].l3)*(lines[ant].l1);
			double	newz = (lines[i].l1)*(lines[ant].l2) - (lines[i].l2)*(lines[ant].l1);
			if ( (not lines[i].ok) or (not lines[ant].ok) or // Bad segment
				(fabs(newz) < EPSILON) ) // Lines too parallel
				result.append(oldPoint);
			else
				{
				int nx = qRound(newx/newz);
				int ny = qRound(newy/newz);
				// Only consider intersection if it is inside
				// a maximum radius circle around the old point
				// (relative to its nearer previous/next point
				// in polyline):
				double dist =
					sqrt((nx-oldPoint.x())*(nx-oldPoint.x()) +
					     (ny-oldPoint.y())*(ny-oldPoint.y()));
				QPointF prevPoint = polyline[list[ant]->index],
					nextPoint = polyline[list[post]->index];
				double minDist =
					qMin(
					sqrt((prevPoint.x()-oldPoint.x())*(prevPoint.x()-oldPoint.x()) +
					(prevPoint.y()-oldPoint.y())*(prevPoint.y()-oldPoint.y())),
					sqrt((nextPoint.x()-oldPoint.x())*(nextPoint.x()-oldPoint.x()) +
					(nextPoint.y()-oldPoint.y())*(nextPoint.y()-oldPoint.y()))
					);
				if(dist < 0.2*minDist)
					result.append(QPoint(nx,ny));
				else
					result.append(oldPoint);
				}
			}
		}
	else 
		{
		// No postprocess, simply store the resulting points in result polyline.
		it = list.begin();
		while(it != list.end())
			{
				result.append(polyline.at((*it)->index));
				it++;
			}
		}

	// Free memory of remaining structure:
	while (!list.isEmpty())
		delete list.takeFirst();

	// Polyline type and direction are the same of the original polyline:
	result.closed = polyline.closed;
	result.direction = polyline.direction;

	// We return the cost of the first non deleted point:
	return return_value;
	}

// Draw
QVPolylineF::QVPolylineF(): QList<QPointF>(),
	closed(false), direction(false)//, dir(0)
	{
	qDebug() << "QVPolylineF()";
	qDebug() << "QVPolylineF() <~ return";
	};

QVPolylineF::QVPolylineF(const QVPolyline &polyline): QList<QPointF>(),
	closed(polyline.closed), direction(polyline.direction)//, dir(polyline.dir)
	{
	foreach(QPoint point, polyline)
		{
		append(QPointF(point));
		}
	qDebug() << "QVPolylineF(const QVPolylineF &)";
	qDebug() << "QVPolylineF(const QVPolylineF &) <~ return";
	};

QVPolylineF::QVPolylineF(const QVPolylineF &polyline): QList<QPointF>(polyline),
	closed(polyline.closed), direction(polyline.direction)//, dir(polyline.dir)
	{
	qDebug() << "QVPolylineF(const QVPolylineF &)";
	qDebug() << "QVPolylineF(const QVPolylineF &) <~ return";
	};


// QPointF	linesIntersection(QPointF a, QPointF b, QPointF c, QPointF d)
// 	// gets the intersection point for lines ab and cd
// 	{
// 	double x1 = a.rx(), x2 = b.rx(), x3 = c.rx(), x4 = d.rx();
// 	double y1 = a.ry(), y2 = b.ry(), y3 = c.ry(), y4 = d.ry();
// 
// 	double denominador = (y4 - y3)*(x2 - x1) - (x4 - x3)*(y2 - y1);
// 	if (denominador == 0)
// 		return QPointF( (int)(b.rx() + c.rx())/2, (int)(b.ry() + c.ry())/2 );
// 
// 	double	ua = (x4 - x3)*(y1 - y3) - (y4 - y3)*(x1 - x3),
// 		ub = (x2 - x1)*(y1 - y3) - (y2 - y1)*(x1 - x3);
// 
// 	QPointF p = QPointF(
// 			(int) (x1 + ua*(x2 - x1) / denominador),
// 			(int) (y1 + ub*(y2 - y1) / denominador)
// 			);
// 	return p;
// 	}



/////////////////////////////////////////

QVPolylineF QVPolylineF::ellipse(uInt n, float x, float y, float maxRadio, float minRadio, float ang)
	{
	QVPolylineF ellipse;
	float w = 2*PI/(n-1);
	float maxArg = (maxRadio+minRadio)/2;
	float minArg = (maxRadio-minRadio)/2;

	for (uInt t = 0; t < n; t++)
		ellipse.append(QPointF (	(uInt) (x + maxArg*cos(ang+w*t) + minArg*cos(ang-w*t)),
						(uInt) (y + maxArg*sin(ang+w*t) + minArg*sin(ang-w*t))
						));
	return ellipse;
	}

QVPolylineF QVPolylineF::rectangle(float x1, float y1, float x2, float y2)
	{
	QVPolylineF rectangle;
	rectangle.append(QPointF( x1, y1 ));
	rectangle.append(QPointF( x1, y2 ));
	rectangle.append(QPointF( x2, y2 ));
	rectangle.append(QPointF( x2, y1 ));
	rectangle.append(QPointF( x1, y1 ));
	return rectangle;
	}

QVPolylineF::operator QVPolyline() const
	{
	QVPolyline polyline;
	foreach(QPointF pointf, *this)
		{
		polyline.append( QPoint(pointf.toPoint()) );
		}
	return polyline;
	}

