#ifndef QMATRIXALGEBRA_H
#define QMATRIXALGEBRA_H

#include <qvector.h>
#include <qvmath/qvmatrix.h>

/// @brief Obtains the <a href="http://en.wikipedia.org/wiki/Singular_value_decomposition">singular value decomposition (SVD)</a> for a matrix.
///
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param M matrix containing matrix to decompose
/// @param U matrix to store resulting matrix U from decomposition
/// @param V matrix to store resulting matrix V from decomposition
/// @param S matrix to store resulting matrix S from decomposition
void singularValueDecomposition(const QVMatrix &M, QVMatrix &U, QVMatrix &V, QVMatrix &S);

/// @brief Obtains the <a href="http://en.wikipedia.org/wiki/LU_decomposition">LU decomposition</a> for a matrix.
///
/// Returms two triangular, upper and lower, matrixes and a permutation matrix (identity if M is invertible). And M = P*L*U
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param M matrix containing matrix to decompose
/// @param L matrix to store resulting matrix L from decomposition
/// @param U matrix to store resulting matrix U from decomposition
/// @param P matrix to store resulting matrix P from decomposition
void LUDecomposition(const QVMatrix &M, QVMatrix &L, QVMatrix &U, QVMatrix &P);

/// @brief Obtains the <a href="http://en.wikipedia.org/wiki/Cholesky_decomposition">Cholesky decomposition</a> for a matrix.
///
/// @todo detailed description should follow 
///
/// @ingroup qvmatrixalgebra
/// @param M matrix containing matrix to decompose (must be positive-definite)
/// @param L matrix to store resulting matrix L from decomposition
void CholeskyDecomposition(const QVMatrix &M, QVMatrix &L);

/// @brief Obtains the <a href="http://en.wikipedia.org/wiki/QR_decomposition">QR decomposition</a> for a matrix.
///
/// @todo detailed description should follow 
///
/// @ingroup qvmatrixalgebra
/// @param M matrix containing matrix to decompose
/// @param Q matrix to store resulting matrix Q from decomposition
/// @param R matrix to store resulting matrix R from decomposition
void QRDecomposition(const QVMatrix &M, QVMatrix &Q, QVMatrix &R);

/// @brief Obtains the <a href="http://en.wikipedia.org/wiki/Pseudoinverse">pseudoinverse</a> for a matrix.
///
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param M matrix containing matrix to obtain pseudoinverse
QVMatrix pseudoInverse(const QVMatrix &M);

/// @brief Obtains the <a href="http://en.wikipedia.org/wiki/Eigendecomposition_of_a_matrix">eigendecomposition</a> for a symetric matrix.
///
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param M symetric matrix to obtain eigendecomposition
/// @param eigVals matrix to store resulting eigenvalues vector 
/// @param eigVecs matrix to store resulting eigenvectors matrix, as row vectors
void eigenDecomposition(const QVMatrix &M, QVVector &eigVals, QVMatrix &eigVecs);

/// @brief Obtains the <a href="http://en.wikipedia.org/wiki/Determinant">determinant</a> of a squared matrix.
///
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param M matrix to obtain the determinant
/// @return the value of the determinant
double determinant(const QVMatrix &M);

/// @brief Obtains the <a href="http://en.wikipedia.org/wiki/Bhattacharyya_distance">Bhattacharyya distance</a> of two gaussian distributions.
///
/// Obtains the Bhattacharyya distance between two Gaussian distributions, given by their
/// mean vectors and covariance matrices.
///
/// @ingroup qvstatistics
/// @param m1 first mean.
/// @param S1 first covariance matrix.
/// @param m2 second mean.
/// @param S2 second covariance matrix.
/// @return computed distance value
double BhattacharyyaDistance(const QVVector &m1, const QVMatrix &S1, const QVVector &m2, const QVMatrix &S2);

/// @brief Solves a system of linear equations using a <a href="http://en.wikipedia.org/wiki/Householder_transformation">Householder transformation</a>.
///
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param A coeficient matrix in the matrix equation form of the problem.
/// @param x vector to store the solution.
/// @param B right-hand side vector in the matrix equation form of the problem.
void solveLinear(const QVMatrix &A, QVVector &x, const QVVector &b);

/// @brief Solves several system of linear equations, using a LU decomposition.
///
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param A coeficient matrix in the matrix equation form of the problem.
/// @param x matrix to store the solutions, each column is a solution.
/// @param B right-hand side matrix in the matrix equation form of the problem, each column is a right-hand side.
void solveLinear(const QVMatrix &A, QVMatrix &X, const QVMatrix &B);

/// @brief Solves several system of linear equations, using a SV decomposition.
///
/// Solves determined and over-determined systems, in the over-determined case the system is solved in the least squares sense.
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param A coeficient matrix in the matrix equation form of the problem.
/// @param x matrix to store the solutions, each column is a solution.
/// @param B right-hand side matrix in the matrix equation form of the problem, each column is a right-hand side.
void solveOverDetermined(const QVMatrix &A, QVMatrix &X, const QVMatrix &B);

/// @brief Solves an <a href="http://en.wikipedia.org/wiki/System_of_linear_equations#Homogeneous_systems">homogeneous linear system</a>.
///
/// @todo detailed description should follow
///
/// @ingroup qvmatrixalgebra
/// @param A coeficient matrix in the matrix equation form of the problem.
/// @param x vector to store the solution.
void solveHomogeneousLinear(const QVMatrix &A, QVector<double> &x);

/// @todo document this function
void solveHomogeneousLinear2(const QVMatrix &A, QVector<double> &x);

/// @todo document this function
double homogLineFromMoments(double x,double y,double xx,double xy,double yy,double &a,double &b,double &c);

/// @todo document and test this function
QVVector regressionLine(const QVMatrix &points);

#endif
