/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvquaternion.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvmath/qvmatrix.h>
#include <qvmath/qvquaternion.h>
#include <QString>

///////////////////////////////////
// Constructors
QVQuaternion::QVQuaternion(): QVVector(4)
	{
	set(0);
	operator[](3) = 1;
	}

QVQuaternion::QVQuaternion(QVVector a, float phi):QVVector(4)
	{
	QVVector a_normal = a * (sin(phi/2.0) / a.norm2());

	operator[](0) = a_normal[0];
	operator[](1) = a_normal[1];
	operator[](2) = a_normal[2];
	operator[](3) = cos(phi/2.0);
	}

QVQuaternion::QVQuaternion(const QVMatrix matrix): QVVector(4)
	{
	operator[](4) = sqrt(1 + matrix(0,0) + matrix(1,1) + matrix(2,2)) / 2;
	operator[](1) = (matrix(1,2) - matrix(2,1)) / (4*operator[](4));
	operator[](2) = (matrix(2,0) - matrix(0,2)) / (4*operator[](4));
	operator[](3) = (matrix(0,1) - matrix(1,0)) / (4*operator[](4));
	}

QVQuaternion::QVQuaternion(const double q1, const double q2, const double q3, const double q4): QVVector(4)
	{
	operator[](0) = q1;
	operator[](1) = q2;
	operator[](2) = q3;
	operator[](3) = q4;
	}

///////////////////////////////

#define TRACKBALLSIZE  (0.8)
float pixelToSphere(const float r, const float x, const float y)
	{
	float d = sqrt(x*x + y*y);
	return (d < r * 0.70710678118654752440)? sqrt(r*r - d*d) : r*r / (2*d);
	}

QVQuaternion QVQuaternion::trackball(const float p1x, const float p1y, const float p2x, const float p2y)
	{
	// Zero rotation
	if (p1x == p2x && p1y == p2y)
		return QVQuaternion();
	
	// First, figure out z-coordinates for projection of P1 and P2 to deformed sphere
	QVVector p1(3), p2(3);
	p1[0] = p1x, p1[1] = p1y, p1[2] = pixelToSphere(TRACKBALLSIZE,p1x,p1y);
	p2[0] = p2x, p2[1] = p2y, p2[2] = pixelToSphere(TRACKBALLSIZE,p2x,p2y);

	// Figure out how much to rotate around that axis.
	float t = (p1 - p2).norm2() / (2.0*TRACKBALLSIZE);
	
	// Avoid problems with out-of-control values...
	if (t > 1.0) t = 1.0;
	if (t < -1.0) t = -1.0;

	return QVQuaternion(p2 ^ p1, 2.0 * asin(t));			
	}

QVQuaternion QVQuaternion::normalizeQuaternion() const
	{
	QVQuaternion q = *this;

	double size = q[0]*q[0] + q[1]*q[1] + q[2]*q[2] + q[3]*q[3];
	for (int i = 0; i < 4; i++)
		q[i] /= size;

	return q;
	}

QVQuaternion QVQuaternion::quaternionProduct(const QVQuaternion &quaternion) const
	{
	Q_ASSERT(size() == 4);
	Q_ASSERT(quaternion.size() == 4);

	QVQuaternion	q1Vector = *this, q2Vector = quaternion;

	QVVector	t1Vector = q1Vector.subVector(0,2),
			t2Vector = q2Vector.subVector(0,2),
			t3Vector = t2Vector ^ t1Vector,
			tfVector = t1Vector * q2Vector[3] + t2Vector * q1Vector[3] + t3Vector;

	QVQuaternion destQuat;

	destQuat[0] = tfVector[0];
	destQuat[1] = tfVector[1];
	destQuat[2] = tfVector[2];
	destQuat[3] = q1Vector[3] * q2Vector[3] - t1Vector * t2Vector;
	
	return destQuat.normalizeQuaternion();
	}

//////////////////////////////////////////////

std::ostream& operator << ( std::ostream &os, const QVQuaternion &quaternion )
	{
	const int size = quaternion.size();

	os << "QVQuaternion [";

	for (int i = 0; i < size; i++)
		os << qPrintable(QString("%1").arg(quaternion[i], -8, 'f', 6)) << " ";

	os << "]" << std::endl;
	return os;
	}

